/*
 * Copyright Ian Burnett 2005, 2006.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 *
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

package com.ianburnett.iilc;


/**
 * Classes should implement this interface if they wish to take part in the
 * running of a display.
 * 
 * <p>There is a well-defined sequence to the creation, running, and deletion of
 * a module using the methods documented here. The module must have a public
 * default constructor. Any native libraries which must be loaded should have
 * the {@link System#loadLibrary(String)} method call in the class static
 * initialiser.</p>
 * 
 * <p>Once the object has been successfully constructed, then the
 * {@link #setOption(String, String)} method is invoked zero or more times,
 * depending on the configuration options provided by the user in the config
 * file.</p>
 * 
 * <p>Once all of the user-options have been set on the object, the object's
 * {@link #init()} method is called. The {@link #setOption(String, String)}
 * method will not be called any more.</p>
 * 
 * <p>Once the object has successfully completed the initialisation routine, then
 * the {@link #start()} method is called. The {@link #init()} will never be
 * called again.</p>
 * 
 * <p>Once the module has entered the running state via the {@link #start()}
 * method, then the module may be called to {@link #stop()}. The module may be
 * called to switch between the started and stopped states multiple times during
 * the lifetime of the object.</p>
 * 
 * <p>Once the module is no longer required, then the {@link #destroy()} method is
 * called. Once this has completed successfully, no further methods will be
 * invoked on this object, and the instance will be eligable for garbage
 * collection.</p>
 * 
 * @author Ian Burnett
 */
public interface Module
{
    public int getHardwareVersionMajor();
    
    public int getHardwareVersionMinor();
    
    public void update();
    
    /**
     * 
     * @param key
     * @param value
     */
    public void setOption(String key, String value);
    
    /**
     * Retrieves the time required to elapse between calls to the
     * {@link #update()} method. The return value from this method is allowed
     * to vary during the lifetime of the module.
     * 
     * @return The time in milliseconds which should elapse before this module
     * should be called to update.
     */
    public int getUpdateInterval();
    
    /**
     * \brief Performs the initial creation of the driver instance.
     *
     * This will be the first function called for a given instance. This function
     * will be called once and once only per device.
     *
     * \param pszPort A string representation of the port which is sufficient to
     * uniquely identify a physical device which this driver can control. Storage
     * is owned by the caller.
     *
     * \param ppDriverInstData pointer to an LP_DRIVER_INSTANCE field which can
     * store context data for the driver module. Storage of the structure pointed
     * to by the return value should be freed during the driver_Dispose() call.
     *
     * \retval int zero on success, non-zero otherwise.
     */
    public void init() throws InvalidConfigurationException;
    
    /** \brief Signals to the implementation to commence communications with the
      * physical device.
      *
      * This function will be called after the driver_Create() function, but before
      * any other functions. There are no restrictions against the driver establishing
      * communications before this function call, but it is an error for the driver
      * to produce callbacks until after this function has exited.
      *
      * \param pDriverInstData Driver instance data as supplied by a call to the 
      * driver_Create() function.
      *
      * \retval int zero on success, non-zero otherwise.
      */
    public void start();
    
    /** \brief Signals to the implementation to cease communications with the
      * physical device.
      *
      * This function will be called before the driver_Dispose() function. There
      * are no restrictions against the driver maintaining communications after this
      * function call, but it is an error for the driver to produce callbacks after
      * this function has exited.
      *
      * It is permissible for there to be multiple driver_Start(), driver_Stop()
      * cycles during the lifetime (driver_Create() to driver_Dispose()) of a driver
      * instance. The implementation should cope with this eventuality.
      *
      * \param pDriverInstData Driver instance data as supplied by a call to the 
      * driver_Create() function.
      *
      * \retval int zero on success, non-zero otherwise.
      */
    public void stop();
    
    /**
     * \brief Performs the final shutdown and cleanup of the driver instance.
     *
     * This function should cleanup any resources it allocated during the
     * driver_Create() function call, or at any point during its lifecycle.
     *
     * \param ppDriverInstData Pointer to an LP_DRIVER_INSTANCE field which
     * has been initialised by a call to the driver_Create() function. It is
     * typically expected that this function sets the contents of this parameter
     * to be NULL upon successful cleanup.
     *
     * \retval int zero on success, non-zero otherwise.
     */
   public void destroy();
   
}
