/*
 * Copyright Ian Burnett 2005, 2006.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 *
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

package com.ianburnett.iilc.display;

import com.ianburnett.iilc.entity.keypad.KeyProvider;

/**
 * 
 * @author Ian Burnett
 */
public interface DisplayProvider
{
    /**
     * 
     * @return
     */
    public int getScreenSizeX();
    
    /**
     * 
     * @return
     */
    public int getScreenSizeY();
    
    /**
     * \brief Writes the supplied text to the device at the location given.
     *
     * \param pDriverInstData Driver instance data as supplied by a call to the 
     * driver_Create() function.
     *
     * \param x Horizontal component of the coordinate to write the text. This 
     * metric is measured in character positions. This is zero-based.
     *
     * \param y Vertical component of the coordinate where to write the text. This
     * metric is measured in character positions. This is zero-based.
     *
     * \param strData A string containing the character data to write. Some displays
     * accept the character '\0' as a valid value, which is why this is not a
     * null-terminated string.
     * 
     * \param strLen Length of the string in characters.
     *
     * \retval int zero on success, non-zero otherwise.
     */
   public void writeText(int x, int y, String data);
   
    /**
     * \brief Clears the screen of the display.
     *
     * \param pDriverInstData Driver instance data as supplied by a call to the 
     * driver_Create() function.
     *
     * \retval int zero on success, non-zero otherwise.
     */
   public void clearScreen(); 

    /**
     * \brief Instructs the display to set the backlight to the specified
     * brightness level.
     *
     * The brightness level is expressed as a percentage - 0 represents off,
     * 100 represents on. A value outside of this range is an error.
     *
     * \param pDriverInstData Driver instance data as supplied by a call to the 
     * driver_Create() function.
     *
     * \param nBacklightValue A percentage value, where 0 &lt;= nBacklightValue &lt;= 100
     *
     * \retval int zero on success, non-zero otherwise.
     */
   public void setBacklight(int nBacklightValue);
   
   /**
     * \brief Instructs the display to set the contrast to the specified ratio.
     *
     * The contrast ratio is expressed as a percentage - 0 represents off,
     * 100 represents on. A value outside of this range is an error.
     *
     * \param pDriverInstData Driver instance data as supplied by a call to the 
     * driver_Create() function.
     *
     * \param nContrastValue A percentage value, where 0 &lt;= nContrastValue &lt;= 100
     *
     * \retval int zero on success, non-zero otherwise.
     */
   public void setContrast(int nContrastValue);

   /**
    * 
    * @return
    */
   public KeyProvider getKeyProvider();
}
