/*
 * Copyright Ian Burnett 2005, 2006.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 *
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

package com.ianburnett.iilc.drivers.crystalfontz;

import java.util.HashMap;
import java.util.Map;

import com.ianburnett.iilc.display.DisplayProvider;
import com.ianburnett.iilc.entity.fan.Fan;
import com.ianburnett.iilc.entity.fan.FanProvider;
import com.ianburnett.iilc.entity.keypad.KeyListener;
import com.ianburnett.iilc.entity.keypad.KeyProvider;
import com.ianburnett.iilc.entity.led.Color;
import com.ianburnett.iilc.entity.led.ControlledLED;
import com.ianburnett.iilc.entity.led.LEDProvider;
import com.ianburnett.iilc.entity.temp.TempSensor;
import com.ianburnett.iilc.entity.temp.TempSensorProvider;

/**
 * This is a 'best-guess' implementation for a CrystalFontz 635
 * display module. I do not own one of these, so have just guessed
 * the implementation details. I don't think we should be too far
 * wrong, however.
 * 
 * @author Ian Burnett
 */
public class CF633 extends CFPacketSerial
        implements FanProvider, LEDProvider, TempSensorProvider, DisplayProvider
{
    /* ================================================================ */
    /* Types of packets which can be sent to and from the display       */
    /* ================================================================ */

    public static final int TYPE_PING                = 0x00;
    public static final int TYPE_GET_VERSION_INFO    = 0x01;
    public static final int TYPE_WRITE_FLASH_AREA    = 0x02;
    public static final int TYPE_READ_FLASH_AREA     = 0x03;
    public static final int TYPE_STORE_AS_BOOT       = 0x04;
    public static final int TYPE_POWER_CONTROL       = 0x05;
    public static final int TYPE_CLEAR_SCREEN        = 0x06;
    public static final int TYPE_SET_LCD_LINE_1      = 0x07;
    public static final int TYPE_SET_LCD_LINE_2      = 0x08;
    public static final int TYPE_SET_CHAR_DATA       = 0x09;
    public static final int TYPE_READ_LCD_MEMORY     = 0x0A;
    public static final int TYPE_SET_CURSOR_POS      = 0x0B;
    public static final int TYPE_SET_CURSOR_STYLE    = 0x0C;
    public static final int TYPE_SET_LCD_CONTRAST    = 0x0D;
    public static final int TYPE_SET_BACKLIGHT       = 0x0E;
    /*                      DEPRECATED               = 0x0F; */
    public static final int TYPE_SET_FAN_REPORTING   = 0x10;
    public static final int TYPE_SET_FAN_POWER       = 0x11;
    public static final int TYPE_READ_DOW_DEVICE     = 0x12;
    public static final int TYPE_SET_TEMP_REPORTING  = 0x13;
    public static final int TYPE_DOW_TRANSACTION     = 0x14;
    public static final int TYPE_SET_LIVE_DISPLAY    = 0x15;
    public static final int TYPE_DIRECT_LCD_CMD      = 0x16;
    public static final int TYPE_SET_KEY_REPORTING   = 0x17;
    public static final int TYPE_READ_KEY_STATE      = 0x18;
    public static final int TYPE_SET_FAN_FAILSAFE    = 0x19;
    public static final int TYPE_SET_FAN_TACH_GLITCH = 0x1A;
    public static final int TYPE_GET_FAN_FAILSAFE    = 0x1B;
    public static final int TYPE_SET_ATX_POWER_SW    = 0x1C;
    public static final int TYPE_MANAGE_WATCHDOG     = 0x1D;
    public static final int TYPE_REPORTING_STATUS    = 0x1E;
    public static final int TYPE_WRITE_DATA          = 0x1F;
    /*                      RESERVED                 = 0x20; */
    public static final int TYPE_SET_BAUD_RATE       = 0x21;
    public static final int TYPE_SET_GPIO            = 0x22;
    public static final int TYPE_GET_GPIO            = 0x23;
    public static final int TYPE_RPT_KEY_ACTIVITY    = 0x80;
    public static final int TYPE_RPT_FAN_SPEED       = 0x81;
    public static final int TYPE_RPT_TEMP_SENSOR     = 0x82;

    /**
     * 
     */
    private int[] _nFanPower = null;
    
    /**
     * 
     */
    private int _hwVersionMajor = -1;
    
    /**
     * 
     */
    private int _hwVersionMinor = -1;
    
    /**
     * 
     */
    private Map _fanMap = null;
    
    /**
     * 
     */
    private Map _tempSensorMap = null;
    
    /**
     * 
     */
    private Map _ledMap;
    
    /**
     * 
     */
    private KeyListener _keyCallback = null;
    
    /* Constructor ------------------------------------------ */

    /**
     * 
     *
     */
    public CF633()
    {
        // We always have at most 4 fans
        this._nFanPower = new int[4];
        
        // Init storage
        _tempSensorMap = new HashMap();
        _fanMap = new HashMap();
        _ledMap = new HashMap();
    }
    
    /* IILCModule ------------------------------------------- */

    /* (non-Javadoc)
     * @see com.ianburnett.iilc.drivers.DriverImpl#getHardwareVersionMajor()
     */
    public int getHardwareVersionMajor()
    {
        return _hwVersionMajor;
    }

    /* (non-Javadoc)
     * @see com.ianburnett.iilc.drivers.DriverImpl#getHardwareVersionMinor()
     */
    public int getHardwareVersionMinor()
    {
        return _hwVersionMinor;
    }

    /* (non-Javadoc)
     * @see com.ianburnett.iilc.drivers.DriverImpl#writeText(int, int, java.lang.String)
     */
    public void writeText(int x, int y, String data)
    {
        Packet p = createPacket(TYPE_WRITE_DATA);
        p.write((byte) x);
        p.write((byte) y);
        for (int i = 0; i < data.length(); i++) {
            p.write((byte) (data.charAt(i) & 0xFF));
        }
        writePacket(p);
    }

    public void update()
    {
        
    }


    public void init()
    {
        // Initialise the superclass first
        super.init();
        
        // Attempt to get the hardware version
        Packet req = createPacket(TYPE_GET_VERSION_INFO);
        Packet resp = writePacket(req);
        char[] data = resp.readData();
        String strResponse = new String(data);
        System.out.println("Connected to [" + strResponse + "]");
        
        req = createPacket(TYPE_RPT_FAN_SPEED);
        req.write((byte) 15);
        resp = writePacket(req);
    }
    
    /* TempProvider ----------------------------------------- */
    
    public void addMonitoredTempSensor(TempSensor sensor)
    {
        _tempSensorMap.put(new Integer(sensor.getID()), sensor);
    }

    /* ScreenProvider --------------------------------------- */
    
    /* (non-Javadoc)
     * @see com.ianburnett.iilc.drivers.DriverImpl#setBacklight(int)
     */
    public void setBacklight(int nBacklightValue)
    {
        Packet p = createPacket(TYPE_SET_BACKLIGHT);
        p.write((byte) nBacklightValue);
        writePacket(p);
    }

    /* (non-Javadoc)
     * @see com.ianburnett.iilc.drivers.DriverImpl#setContrast(int)
     */
    public void setContrast(int nContrastValue)
    {
        Packet p = createPacket(TYPE_SET_LCD_CONTRAST);
        p.write((byte) nContrastValue);
        writePacket(p);
    }

    public int getScreenSizeX()
    {
        return 16;
    }

    public int getScreenSizeY()
    {
        return 2;
    }
    
    /* (non-Javadoc)
     * @see com.ianburnett.iilc.drivers.DriverImpl#clearScreen()
     */
    public void clearScreen()
    {
        writePacket(createPacket(TYPE_CLEAR_SCREEN));
    }

    /* LEDProvider ------------------------------------------ */
    
    public void addMonitoredLED(ControlledLED led)
    {
        _ledMap.put(new Integer(led.getID()), led);
    }

    /* (non-Javadoc)
     * @see com.ianburnett.iilc.drivers.DriverImpl#setLED(int, int, int, int)
     */
    public void setLED(ControlledLED led, Color color)
    {
        int id = led.getID();
        
        // Ignore the blue content of the colour

        // Write out the green component
        Packet p = createPacket(TYPE_SET_GPIO);
        p.write((byte) (11 - (id * 2)));
        p.write((byte) color.g);
        writePacket(p);

        // Write out the red component
        p = createPacket(TYPE_SET_GPIO);
        p.write((byte) (12 - (id * 2)));
        p.write((byte) color.r);
        writePacket(p);
    }

    /* FanProvider ------------------------------------------ */
    
    public void addMonitoredFan(Fan f)
    {
        _fanMap.put(new Integer(f.getID()), f);
    }

    /* (non-Javadoc)
     * @see com.ianburnett.iilc.drivers.DriverImpl#setFanPower(int, int)
     */
    public void setFanPower(int nFan, int nPower)
    {
        // Save the fan setting
        _nFanPower[nFan] = nPower;
        
        // Create a new packet
        Packet p = createPacket(TYPE_SET_FAN_POWER);
        
        // Write all fan power data out
        for (int i = 0; i < _nFanPower.length; i++) {
            p.write((byte) (_nFanPower[i] & 0xFF));
        }
    
        // Write the packet out
        Packet r = writePacket(p);
    }

    /* Internal methods ------------------------------------- */
    
    /**
     * 
     */
    private void fanSpeedReport(Packet p)
    {
        // Pulses per revolution
        int nPPR = 2;
        
        // Read the data from the packet
        char[] data = p.readData();
        
        // Pull the data into useful numbers
        int fan = data[0];
        int rpm = 0;
        int nFanTachCycles = data[1];
        int rawTimerTicks = (data[3] << 8) + data[2];
        
        if (nFanTachCycles < 3) {
            rpm = Fan.SPEED_STOPPED;
        }
        else if (nFanTachCycles < 4) {
            rpm = Fan.SPEED_TOO_LOW;
        }
        else if (nFanTachCycles == 0xFF) {
            rpm = Fan.SPEED_TOO_HIGH;
        }
        else {
            rpm = (((27692308 / nPPR) * (nFanTachCycles - 3)) / rawTimerTicks);
        }

        // Lookup the Fan object to receive the report
        Fan objFan = (Fan) _fanMap.get(new Integer(fan));
        if (objFan == null) {
            System.out.println("No callback for fan " + fan);
            return; // No callback required
        }
        
        // Perform the callback
        objFan.reportFanSpeed(rpm);
    }

    private void keyReport(Packet p)
    {
        // Early exit if nothing to report to
        if (_keyCallback == null) {
            return;
        }
        
        // Read the data from the packet
        char[] data = p.readData();
        
        // Decode this into the correct key
        int key = 0;
        switch (data[0]) {
        case 1:     case 7:     key = KeyListener.KEY_UP;       break;
        case 2:     case 8:     key = KeyListener.KEY_DOWN;     break;
        case 3:     case 9:     key = KeyListener.KEY_LEFT;     break;
        case 4:     case 10:    key = KeyListener.KEY_RIGHT;    break;
        case 5:     case 11:    key = KeyListener.KEY_YES;      break;
        case 6:     case 12:    key = KeyListener.KEY_NO;       break;
        }
       
        // Decode this into press/release information 
        switch (data[0]) {
        case 1:     case 2:     case 3:
        case 4:     case 5:     case 6:
            _keyCallback.reportKeyDown(key);
            break;
        
        case 7:     case 8:     case 9:
        case 10:    case 11:    case 12:
            _keyCallback.reportKeyRelease(key);
            break;
        }
    }
    
    private void temperatureReport(Packet p)
    {
        // Read the data from the packet
        char[] data = p.readData();
        
        // Sensor is index 0
        int sensor = data[0];
        
        // Lookup the TempSensor object to receive the report
        TempSensor objSensor = (TempSensor) _tempSensorMap.get(new Integer(sensor));
        if (objSensor == null) {
            System.out.println("No callback for sensor " + sensor);
            return; // No callback required
        }
        
        // Temp is @ index 1 & 2
        int rawTemp = (data[2] << 8) + data[1];
        
        // Index 3 is the DOW CRC status
        if (data[3] == 0) {
            System.out.println("Bad DOW CRC status");
            return;
        }
        
        // Calculate the temp
        double degc = rawTemp / 16.0;
        
        // Perform the callback
        objSensor.reportTemperature(degc);
    }

    protected void asyncPacketReceived(Packet p)
    {
        // Decide what to do with this
        switch ( p.getPacketType() ) {

        case TYPE_RPT_TEMP_SENSOR:
            temperatureReport(p);
            break;
            
        case TYPE_RPT_FAN_SPEED:
            fanSpeedReport(p);
            break;
            
        case TYPE_RPT_KEY_ACTIVITY:
            keyReport(p);
            break;

        default:
            break;
        }
    }
    
    protected int getPacketClass(Packet p)
    {
        return (p.getPacketType() & 0xC0) >> 6;
        
    }

    protected Packet createPacket()
    {
        return new CF633Packet();
    }

    protected Packet createPacket(int type)
    {
        return new CF633Packet(type);
    }
    
    /**
     * 
     * @author Ian Burnett
     */
    private class CF633Packet extends Packet
    {
        private static final int PACKET_MAX_DATA_LEN = 18;
        
        private CF633Packet()
        {
            super(PACKET_MAX_DATA_LEN);
        }
        
        private CF633Packet(int type)
        {
            super(PACKET_MAX_DATA_LEN, type);
        }
        
    }

    public KeyProvider getKeyProvider()
    {
        // TODO Auto-generated method stub
        return null;
    }
}



