/*
 * Copyright Ian Burnett 2005, 2006.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 *
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

package com.ianburnett.iilc.drivers.crystalfontz;

import sun.rmi.runtime.GetThreadPoolAction;

import com.ianburnett.iilc.InvalidConfigurationException;
import com.ianburnett.iilc.Module;
import com.ianburnett.iilc.impl.ModuleImpl;

/**
 * 
 *
 * @author Ian Burnett
 */
abstract class CFPacketSerial extends ModuleImpl implements Runnable
{
    /**
     * 
     */
    private Packet _RxPacket = null;
    
    /**
     * 
     */
    private final Object _RxPacketLock = new Object();

    /**
     * 
     */
    private boolean _bStarted = false;
    
    /**
     * 
     */
    private boolean _bRunning = false;
    
    /**
     * Lock object used to ensure we only ever attempt to read
     * from the display on one thread at a time.
     */
    private final Object _readLock = new Object();
    
    /**
     * Lock object used to ensure we only ever attempt to write
     * to the display on one thread at a time.
     */
    private final Object _writeLock = new Object();

    /** 
     * The port this serial device is connecting to.
     */
    private String _strPort = null;
    
    /**
     * Used by the native code implementation to store the native handle
     * which is required at the lowest level. We use a long here as it's
     * quite possibly a pointer we get back.
     */
    private long _handle = 0;

    /**
     * Used by the native code implementation to specify what signalling
     * rate we should use to communicate with the physical device. Default
     * rate is 115200 bits/second, but can be changed via the "baud"
     * option. 
     */
    private int _nBaud = 115200;
    
    /**
     * 
     */
    private Thread _thrRx = null;

    /**
     * Static initialiser which loads the correct native library.
     */
    static
    {
        System.loadLibrary("cfontz");
    }
    
    /* Module ----------------------------------------------- */
    
    /**
     * 
     */
    public void init()
    {
        // Find out what port we've been set to
        _strPort = getOption("port");
        if (_strPort == null) {
            // Throw an error
            System.out.println("NO PORT!!!");
        }

        // Check if a baud rate has been set
        String strBaud = getOption("baud");
        if (strBaud != null) {
            try {
                _nBaud = Integer.parseInt(strBaud);
            }
            catch (NumberFormatException nfe) {
                // Put out a warning
            }
        }
        
        // Call to the low-level init routine
        initialiseDevice(_strPort);
        if (_handle == 0) {
            throw new RuntimeException("Failed to init hardware");
        }
        
        // Need to wait for a while until this device inits itself
        try {
            Thread.sleep(350);
        }
        catch (InterruptedException ie) {
            // Happy to ignore in here
        }

        // Create a new receive thread based on this object
        _thrRx = new Thread(this);
        
        // Start the thread now, even though we're not going to
        // dispatch any async responses
        _bRunning = true;
        _thrRx.start();
    }
    
    /**
     * @see Module#start()
     */
    public void start()
    {
        // Start the superclass first
        super.start();
        
        // Flag this as started
        _bStarted = true;
    }

    /**
     * @see Module#getUpdateInterval()
     */
    public int getUpdateInterval()
    {
        return 10000;
    }
    
    /**
     * @see Module#update()
     */
    public void update()
    {
        
    }
    
    /**
     * @see Module#stop()
     */
    public void stop()
    {
        // Flag this is as stopped
        _bStarted = false;
        
        // Stop the superclass
        super.stop();
    }

    public void destroy()
    {
        // Flag this is as stopped
        _bRunning = false;

        // Wait for the Rx thread to shutdown
        try {
            System.out.println("Waiting for Rx thread to shut down");
            _thrRx.join();
        }
        catch (InterruptedException ie) {
            
        }
        
        // Destroy the superclass
        super.destroy();
    }
    
    /* Runnable --------------------------------------------- */
    
    /**
     * 
     */
    public void run()
    {
        // Loop until we're told to stop
        while (_bRunning) {
            rxLoop();
        }
    }
    
    /* Routines for subclasses ------------------------------ */

    protected Packet writePacket(Packet p)
    {
        // The packet to return as a reply
        Packet reply = null;
        
        // Flatten to a simple buffer
        byte[] data = p.flatten();
        
        // Try a few times to write to the device
        for (int i = 0; i < 5; i++) {
            
            // Write the data
            writeData(data);
    
            synchronized (_RxPacketLock) {
                
                try {
                    // Wait for a maximum of 250ms (which is what the PDF user doc says)
                    _RxPacketLock.wait(250);
                }
                catch (InterruptedException ie) {
                    
                }
                
                // Did we receive one?
                if (_RxPacket != null) {
                    // Grab the reply packet and clear
                    reply = _RxPacket;
                    _RxPacket = null;
                    break;
                }
            }
        }

        // Did we get a reply? If not, create a time-out packet for the user
        return reply != null ? reply : createPacket(0xFF);
    }

    /* Internal routines ------------------------------------ */
    
    /**
     * 
     */
    private boolean rxLoop()
    {
        Integer errno = new Integer(0);
        int nBytesRead = 0, nBytesAvailable = 0, nBytesNew = 0;
        byte[] buf = new byte[0];
    
        do {
            // Attempt to read some data from the device
            byte[] newBuf = readData(errno);
            nBytesNew = newBuf.length;
            
            // Anything available
            if (nBytesAvailable <= 0 && nBytesNew <= 0) {
                try {
                    // Snooze for 10ms while we wait for something to arrive
                    Thread.sleep(10);
                }
                catch (InterruptedException ie) { }
                continue;
            }
            else {
                // Copy into the master buffer
                byte[] tmp = new byte[nBytesAvailable + nBytesNew];
                System.arraycopy(buf, 0, tmp, 0, nBytesAvailable);
                System.arraycopy(newBuf, 0, tmp, nBytesAvailable, nBytesNew);
                buf = tmp;
            }
            
            // We now have new bytes in the buffer
            nBytesAvailable += nBytesNew;
            
            // Nothing read from the buffer yet
            nBytesRead = 0;
            
            // Try to inflate read data into a packet
            Packet p = createPacket();
            nBytesRead = p.populate(buf);
            
            // Remove any bytes that have been read
            if (nBytesRead > 0) {
                nBytesAvailable -= nBytesRead;
                byte[] tmp = new byte[nBytesAvailable];
                System.arraycopy(buf, nBytesRead, tmp, 0, nBytesAvailable);
                buf = tmp;
            }
            
            // Did we successfully read the packet?
            if ( ! p.isValid() ) {
                try {
                    // Probably need to wait for more data
                    Thread.sleep(10);
                }
                catch (InterruptedException ie) { }
                continue;
            }
            
            // What sort of packet is this?
            switch ( getPacketClass(p) ) {
            
            // Async report from device to app
            case Packet.CLASS_NORMAL_REPORT:
                if ( _bStarted ) {
                    asyncPacketReceived(p);
                }
                break;
                
            // Both normal and error responses to be handed back to waiting caller
            case Packet.CLASS_NORMAL_RESPONSE:
            case Packet.CLASS_ERROR_RESPONSE:
                
                synchronized (_RxPacketLock) {
                    
                    // Stash this packet away for someone else to pick up
                    
                    // Set the reply packet and forget about it
                    _RxPacket = p;
                    p = null;
                    
                    // Inform anyone waiting on this object
                    _RxPacketLock.notify();
                }
                break;
                
            // Shouldn't get this packet received!
            case Packet.CLASS_NORMAL_REQUEST:
                break;

            // Broken!!
            default:
                break;
            }
            
        } while (nBytesAvailable > 0);

        return true;
    }    
    
    private String strerr(Integer errno)
    {
        return strerror(errno.intValue());
    }
    
    private String strerror(int errno)
    {
        return null;
    }
    
    /**
     * Java routine which coordinates writing data to the underlying device.
     * 
     * @param buf
     */
    private void writeData(byte[] buf)
    {
        synchronized (_writeLock) {
            writeData(this._handle, buf);
        }
    }
    
    /**
     * Java routine which cordinates reading data from the underlying device.
     *  
     * @param errno
     * 
     * @return
     */
    private byte[] readData(Integer errno)
    {
        synchronized (_writeLock) {
            return readData(this._handle, errno);
        }
    }

    /**
     * Provides the ability to call down an initialise the device at the
     * lowest level.
     * 
     * @param port The port to open in a machine-dependant format.
     * 
     * @return An opaque handle to be used by the {@link #writeData(byte[])}
     * and {@link #readData(Integer)} routines. 
     */
    private native void initialiseDevice(String port);

    /**
     * Provides the ability to call down and clean up the device at the
     * lowest level.
     * 
     * @param handle The stored handle as returned from
     * {@link #initialiseDevice(String)}
     */
    private native void destroyDevice(long handle);

    /**
     * Low-level implementation detail.
     * @param handle
     * @param errno
     * @return
     */
    private native byte[] readData(long handle, Integer errno);

    /**
     * Native routine which handles writing the data to the underlying
     * device. This routine does pass in data which could potentially
     * be obtained through the instance handle, but the native code looks
     * cleaner when you just do it this way.
     * 
     * @param handle The stored handle as returned from
     * {@link #initialiseDevice(String)}.
     * @param buf The data buffer to write. 
     * 
     * @return The number of bytes which were successfully written.
     */
    private native long writeData(long handle, byte[] buf);

    /**
     * Template method provided for future use.
     * 
     * @return
     */
    protected static int getLastError()
    {
        return 0;
    }
    
    /**
     * Template method provided for future use.
     * 
     * @return
     */
    protected static String getLastErrorMessage()
    {
        return "";
    }
    
    /**
     * Called from the native code as a means of passing back any
     * extended error information as supplied by the OS.
     * 
     * There is no reason why this should be an instance method, but
     * there appears to be a bug in the IBM Windows JVM. I think this
     * defect only surfaces when you attempt to call a static Java
     * method from C on a thread which is not 'main'. Calling an
     * instance method appears to work fine. The realistic overhead
     * of the extra couple of parameters is hardly going to be a burden
     * in this system though. :-)
     *  
     * @param nLastError
     * @param strErrorString
     */
    protected void setLastError(long nLastError, String strErrorString)
    {
        // Get current thread context and store error code
        
        // For now, just dump out
        System.out.println("nLastError : " + nLastError + ", str : " + strErrorString);
    }
    
    /**
     * 
     * @param p
     */
    protected abstract void asyncPacketReceived(Packet p);
    
    protected abstract int getPacketClass(Packet p);
    
    protected abstract Packet createPacket();
    
    protected abstract Packet createPacket(int type);
}

