/*
 * Copyright Ian Burnett 2005, 2006.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 *
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

package com.ianburnett.iilc.entity.fan;

import com.ianburnett.iilc.entity.Entity;

/**
 * Represents a fan in the hardware whose status can be read, but not written. 
 * 
 * @author Ian Burnett
 */
public interface Fan extends Entity
{
    /**
     * The fan is in the stopped state.
     */
    public static int SPEED_STOPPED = 0;

    /**
     * The fan spin speed is unknown at this time.
     */
    public static int SPEED_UNKNOWN = -1;
    
    /**
     * The fan is spinning too slowly for the hardware to measure.
     */
    public static int SPEED_TOO_LOW = -2;
    
    /**
     * The fan is spinning at a rate too high for the hardware to measure.
     */
    public static int SPEED_TOO_HIGH = -3;
    
    /**
     * Returns the power level which the fan is currently set to.
     * This value is expressed as a percentage.
     * 
     * @return An integer in the range 0 to 100 inclusive.
     */
    public int getCurrentPowerLevel();
    
    /**
     * Returns the current fan speed in RPM. This is normally represented
     * by a simple int value. This method may also return one of the special
     * SPEED_* values.
     * 
     * @return A int represeting the speed of the fan in RPM, or a special
     * value.
     * 
     * @see #SPEED_STOPPED
     * @see #SPEED_TOO_LOW
     * @see #SPEED_TOO_HIGH
     * @see #SPEED_UNKNOWN
     */
    public int getCurrentRPM();
    
    /**
     * Used by the provider to report the current speed of the fan 
     * 
     * @param rpm Current speed of the fan, or one of the special SPEED_*
     * values.
     * 
     * @see #SPEED_STOPPED
     * @see #SPEED_TOO_LOW
     * @see #SPEED_TOO_HIGH
     * @see #SPEED_UNKNOWN
     */
    public void reportFanSpeed(int rpm);
}
