/*
 * Copyright Ian Burnett 2005, 2006.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 *
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

package com.ianburnett.iilc.modules;

import com.ianburnett.iilc.Module;
import com.ianburnett.iilc.display.DisplayController;
import com.ianburnett.iilc.display.TextArea;
import com.ianburnett.iilc.display.Window;
import com.ianburnett.iilc.entity.temp.TempSensor;
import com.ianburnett.iilc.entity.temp.TempSensorListener;
import com.ianburnett.iilc.impl.ModuleImpl;

/**
 * Simple module which is connected to a single temperature sensor
 * and outputs the current temperature to the display. 
 * 
 * @author Ian Burnett
 */
public class FanSpeedDisplay extends ModuleImpl
        implements TempSensorListener, DisplayController
{
    /**
     * The sensor in use.
     */
    private TempSensor _sensor; 
    
    /**
     * Shortcut to the name of the sensor.
     */
    private String _strSensorName;
    
    /**
     * The text area this module should write to.
     */
    private TextArea _textArea;

    /**
     * Set to true if we've requested the temperature in old money.
     */
    private boolean _bInFarenheit = false;
    
    /* MAIN PROCESSING LOGIC ------------------------------------------------ */
    
    /**
     * @see Module#getUpdateInterval()
     */
    public void update()
    {
        // Get the current temp in Celcius
        String strSuffix = "C";
        double temp = _sensor.getCurrentTemperature();
        
        // Convert to Farenheit?
        if (_bInFarenheit) {
            strSuffix = "F";
            temp = ((9.0 * temp) / 5.0) + 32.0;
        }
        
        // Format to 1 d.p. (add 0.05 for rounding)
        Double dTimesTen = new Double((temp + 0.05) * 10.0);
        temp = dTimesTen.intValue() / 10.0;

        // Send to the display
        _textArea.setText(0, _strSensorName + ":" + temp + strSuffix);
    }

    /* INITIALISATION ROUTINES ---------------------------------------------- */
    
    /**
     * @see Module#init()
     */
    public void init()
    {
        // Init superclass first
        super.init();
        
        // Decide if this is to be shown in C or F
        String strUnit = getOption("unit");
        if (strUnit != null && strUnit.trim().equalsIgnoreCase("f")) {
            _bInFarenheit = true;
        }
    }
    
    /**
     * @see TempSensorListener#addTempSensor(TempSensor)
     */
    public void addTempSensor(TempSensor sensor)
    {
        // Assume only one sensor
        _sensor = sensor;
        
        // Store the name
        _strSensorName = sensor.getName();
    }

    /**
     * @see DisplayController#addDisplayWindow(Window)
     */
    public void addDisplayWindow(Window window)
    {
        // Get dimensions
        int w = window.getWidth(), h = window.getHeight();
        
        // TODO Check this is wide enough
        
        // Initialise to have a single line of text at (0,0)
        _textArea = window.createTextArea(0, 0, w, h);
    }

    /**
     * This module only needs updating every 2 seconds.
     * 
     * @see Module#getUpdateInterval()
     */
    public int getUpdateInterval()
    {
        return 2000;
    }
    
}
