/*
 * Copyright Ian Burnett 2005, 2006.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 *
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

package com.ianburnett.iilc.modules;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.omg.CORBA._PolicyStub;

import com.ianburnett.iilc.Module;
import com.ianburnett.iilc.entity.fan.ControlledFan;
import com.ianburnett.iilc.entity.fan.Fan;
import com.ianburnett.iilc.entity.fan.FanController;
import com.ianburnett.iilc.entity.fan.FanListener;
import com.ianburnett.iilc.entity.led.Color;
import com.ianburnett.iilc.entity.led.ControlledLED;
import com.ianburnett.iilc.entity.led.LEDController;
import com.ianburnett.iilc.entity.temp.TempSensor;
import com.ianburnett.iilc.entity.temp.TempSensorListener;
import com.ianburnett.iilc.impl.ModuleImpl;

/**
 * Fairly simple class to control one or more fans based on the
 * inputs received from one or more temperature sensors.
 * 
 * This module also has the option of displaying current status
 * on an LED which changes colour according to the fan output
 * power levels.
 * 
 * @author Ian Burnett
 */
public class Fans extends ModuleImpl
        implements FanController, TempSensorListener, LEDController
{
    /**
     * Integer constant defined to indicate the fans are currently off.
     */
    private static final int FAN_STATE_OFF = 0;
    
    /**
     * Integer constant defined to indicate the fans are currently running at
     * normal speed.
     */
    private static final int FAN_STATE_ON = 1;
    
    /**
     * Integer constant defined to indicate the fans are currently running at
     * high speed in emergency mode.
     */
    private static final int FAN_STATE_HIGH = 2;

    /**
     * Colour used on the LED to indicate the fans are running in emergency mode.
     */
    private static final Color COLOR_RED = new Color(50, 0, 0);
    
    /**
     * Colour used on the LED to indicate the fans are running in normal mode.
     */
    private static final Color COLOR_AMBER = new Color(50, 50, 0);
    
    /**
     * Colour used on the LED to indicate the fans are currently stopped.
     */
    private static final Color COLOR_GREEN = new Color(0, 50, 0);
            
    /**
     * The collection of fans we have under our control.
     */
    private Set _fanControlList = new HashSet();
    
    /**
     * The collection of temperature sensors we receive reports for.
     */
    private Set _tempReportList = new HashSet(); 
    
    /**
     * A collection of PowerRange objects which represents the highest and lowest
     * values each fan will be set to when set to that state.
     */
    private Map _fanRange = new HashMap();
    
    /**
     * A collection of TempRange object which represents the trigger points for
     * each temperature range.
     */
    private Map _tempRange = new HashMap();
    
    /**
     * Internal flag used to signify how the fans. This will be one of the
     * FAN_STATE_* values.
     * 
     * @see #FAN_STATE_OFF
     * @see #FAN_STATE_ON
     * @see #fAN_STATE_HIGH
     */
    private int _nFanState = FAN_STATE_OFF;
    
    /**
     * The LED which may be used to display the current power output level.
     */
    private ControlledLED _led = null;
    
    /**
     * @see FanController#addControlledFan(ControlledFan)
     */
    public void addControlledFan(ControlledFan fan)
    {
        _fanControlList.add(fan);
    }

    /**
     * @see TempSensorListener#addTempSensor(TempSensor)
     */
    public void addTempSensor(TempSensor sensor)
    {
        _tempReportList.add(sensor);
    }

    /**
     * @see LEDController#addControlledLED(ControlledLED)
     */
    public void addControlledLED(ControlledLED led)
    {
        // Only expecting one LED!
        this._led = led;
    }
    
    /**
     * @see Module#init()
     */
    public void init()
    {
        // Setup the rest of the object first
        super.init();
        
        // Assume the fans are stopped
        _nFanState = FAN_STATE_OFF;
        
        // Loop through each fan, processing any options
        Iterator iterFans = _fanControlList.iterator();
        while (iterFans.hasNext()) {
        
            // Get the fan and name
            ControlledFan fan = (ControlledFan) iterFans.next();
            String name = fan.getName();
            
            // Look for a low & high fan value
            String strLo = getOption("fan_" + name + "_Off");
            String strHi = getOption("fan_" + name + "_On");

            // Create a new power range
            int nLo = 0, nHi = 100;
            
            try {
                if (strLo != null) {
                    nLo = Integer.parseInt(strLo);
                }
            }
            catch (NumberFormatException nfe) { }
            
            try {
                if (strHi != null) {
                    nHi = Integer.parseInt(strHi);
                }
            }
            catch (NumberFormatException nfe) { }
            
            // Create range and store
            _fanRange.put(name, new PowerRange(nLo, nHi));
        }
        
        // Loop through each temperature sensor, processing any options
        Iterator iterTemp = _tempReportList.iterator();
        while (iterTemp.hasNext()) {
         
            // Get the temp sensor and name
            TempSensor sensor = (TempSensor) iterTemp.next();
            String name = sensor.getName();
            
            // Look for a threshold and high value
            String strLim = getOption("temp_" + name + "_Lim");
            String strHi  = getOption("temp_" + name + "_Hi");
            String strLo  = getOption("temp_" + name + "_Lo");
            if (strLim == null || strHi == null || strLo == null) {
                continue;
            }
            
            // Create a new temp range
            double dLim = 0.0, dHi = 0.0, dLo = 0.0;
            
            try {
                dLim = Double.parseDouble(strLim);
                dHi  = Double.parseDouble(strHi);
                dLo  = Double.parseDouble(strLo);
            }
            catch (NumberFormatException nfe) {
                // Ignore and continue
                continue;
            }
            
            // Create range and store
            _tempRange.put(name, new TempRange(dLo, dHi, dLim));
        }
    }
    
    /**
     * Called periodically to update our module
     */
    public void update()
    {
        int nAction = FAN_STATE_OFF;
        
        // Check none of the temperatures hasn't strayed too high
        Iterator iterTemps = _tempReportList.iterator();
        while (iterTemps.hasNext()) {
            
            // Get the sensor and the temperature
            TempSensor sensor = (TempSensor) iterTemps.next();
            String name = sensor.getName();
            double t = sensor.getCurrentTemperature();
            
            // Look this up in the master list
            TempRange range = (TempRange) _tempRange.get(name); 
            if (range == null) {
                // OK to ignore
                System.out.println("No temp range for " + name);
                continue;
            }

            // t >= Lim
            if (t >= range._lim) {
                // Switch on the fans to full power immediately
                nAction = FAN_STATE_HIGH;
                break;
            }

            // Lo <= t < Lim
            else if (t >= range._lo) {
                
                if (t < range._hi) {
                    
                    // Lo <= t < Hi
                    switch (_nFanState) {
                    
                    // Now OK to reduce from full power                    
                    case FAN_STATE_HIGH:
                        nAction = FAN_STATE_ON;
                        break;
                        
                    // Leave alone if currently running
                    case FAN_STATE_ON:
                        nAction = FAN_STATE_ON;
                        break;

                    // If currently off, make sure no-one else wants this on
                    case FAN_STATE_OFF:
                        nAction = (nAction > FAN_STATE_OFF ? nAction : FAN_STATE_OFF);
                        break;
                    }
                }
                else {
                    // Hi <= t < Lim
                    switch (_nFanState) {
                    
                    // Leave running at max until we drop below the Hi mark
                    case FAN_STATE_HIGH:
                        nAction = FAN_STATE_HIGH;
                        break;

                    // Leave on if currently running
                    case FAN_STATE_ON:
                        nAction = FAN_STATE_ON;
                        break;
                    
                    // Fans should be at least on
                    case 0:
                        nAction = FAN_STATE_ON;
                        break;
                    }
                }
            }
            
            // t < Lo
            else {
                // Switch off only if no-one else wants it on
                nAction = (nAction > FAN_STATE_OFF ? nAction : FAN_STATE_OFF);
                continue;
            }
        }

        // Store the fan state
        _nFanState = nAction;
        
        // Display output on LED if needed
        if (_led != null) {
            switch (_nFanState) {
            case FAN_STATE_OFF:  _led.setColour(COLOR_GREEN); break;
            case FAN_STATE_ON:   _led.setColour(COLOR_AMBER); break;
            case FAN_STATE_HIGH: _led.setColour(COLOR_RED);   break;
            }
        }

        // Skim through all fans, updating as required
        Iterator iterFans = _fanControlList.iterator();
        while (iterFans.hasNext()) {
            
            // Get fan and its name
            ControlledFan fan = (ControlledFan) iterFans.next();
            String name = fan.getName();
            
            // Get the allowed range of this fan
            PowerRange range = (PowerRange) _fanRange.get(name);
            int lo = range._lo, hi = range._hi;
            
            // Decide what to do
            switch (nAction) {
            
            case FAN_STATE_OFF:
                fan.setFanPowerLevel(lo);
                break;
                
            case FAN_STATE_ON:
                fan.setFanPowerLevel(hi);
                break;
                
            case FAN_STATE_HIGH:
                fan.setFanPowerLevel(100);
                break;
            }
        }
    }

    private class TempRange
    {
        private double _lo;
        
        private double _hi;
        
        private double _lim;
        
        private TempRange(double lo, double hi, double lim)
        {
            this._lo = lo;
            this._hi = hi;
            this._lim = lim;
        }
    }

    private class PowerRange
    {
        private int _hi;
    
        private int _lo;
        
        private PowerRange(int lo, int hi)
        {
            this._lo = lo;
            this._hi = hi;
        }
    }
}
