/*
 * Copyright Ian Burnett 2005, 2006.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 *
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

package com.ianburnett.iilc.modules;

import com.ianburnett.iilc.Module;
import com.ianburnett.iilc.entity.led.Color;
import com.ianburnett.iilc.entity.led.ControlledLED;
import com.ianburnett.iilc.entity.led.LEDController;
import com.ianburnett.iilc.impl.ModuleImpl;

/**
 * Provides a simple "heartbeat" indicator via an LED. This
 * module by default requests an update to provide a flash rate
 * of 1Hz, with a duty cycle of 10%. These can be adjusted by
 * setting the module options "interval" (in milliseconds) and
 * "duty" (as a percentage). It assumes only one LED is to be
 * controlled.
 * 
 * @author Ian Burnett
 */
public class Heartbeat extends ModuleImpl implements LEDController
{
    /** 
     * The LED we control.
     */
    private ControlledLED _led = null;

    /**
     * Current state of the LED.
     */
    private boolean _bIsOn = false;
    
    /**
     * Length of time in milliseconds we are to be switched on for.
     */
    private int _nOnTime = 100;

    /**
     * Length of time in milliseconds we are to be switched off for.
     */
    private int _nOffTime = 900;
    
    /**
     * @see Module#init()
     */
    public void init()
    {
        // Default of 1000ms cycle time @ 10% on
        int nInterval = 1000, nDutyCycle = 10;
        
        try {
            // Attempt to get the frequency we should be updated with
            String strInterval = getOption("interval");
            nInterval = Integer.parseInt(strInterval);
            if (nInterval < 1) {
                nInterval = 1000;
            }
        }
        catch (NumberFormatException nfe) {
            // Ignore - use default
        }
        
        try {
            // Same for duty cycle
            String strDuty = getOption("duty");
            nDutyCycle = Integer.parseInt(strDuty);
            if (nDutyCycle < 0 || nDutyCycle > 100) {
                nDutyCycle = 10;
            }
        }
        catch (NumberFormatException nfe) {
            // Ignore - use default
        }
        
        // Calculate the on and off periods
        _nOnTime  = (nInterval * nDutyCycle) / 100;
        _nOffTime = nInterval - _nOnTime;
    }
    
    /**
     * This module only controls a single LED.
     * 
     * @see LEDController#addControlledLED(ControlledLED)
     */
    public void addControlledLED(ControlledLED led)
    {
        _led = led;
    }

    /**
     * @see Module#getUpdateInterval()
     */
    public int getUpdateInterval()
    {
        return _bIsOn ? _nOnTime : _nOffTime; 
    }
    
    /**
     * @see Module#update()
     */
    public void update()
    {
        // Assume this is to be switched off
        Color c = new Color(0, 0, 0);
        
        // Should we switch it on or off?
        c.g = _bIsOn ? 0 : 100;
        _bIsOn = ! _bIsOn;

        // Update LED
        _led.setColour(c);
    }
}
