/*
 * Copyright Ian Burnett 2005, 2006.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 *
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

package com.ianburnett.iilc.modules;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.ianburnett.iilc.Module;
import com.ianburnett.iilc.entity.fan.Fan;
import com.ianburnett.iilc.entity.fan.FanListener;
import com.ianburnett.iilc.entity.temp.TempSensor;
import com.ianburnett.iilc.entity.temp.TempSensorListener;
import com.ianburnett.iilc.impl.ModuleImpl;

/**
 * Module implementation which writes a log of all data to
 * a CSV text file.
 * 
 * @author Ian Burnett
 */
public class Log extends ModuleImpl implements FanListener, TempSensorListener
{
    /**
     * The output stream all log data should be written to.
     */
    private PrintStream _psLog = null;
    
    /**
     * List of fans we are monitoring. This must be a list as we wish to
     * preserve ordering when we iterate.
     */
    private List _fans = new ArrayList();
    
    /**
     * List of temperature sensors we are monitoring. This must be a list
     * as we wish to preserve ordering when we iterate.
     */
    private List _sensors = new ArrayList();
    
    /**
     * @see Module#init()
     */
    public void init()
    {
        // Determine the log location
        String strFileName = getOption("file");
        
        // Check this has been provided
        if (strFileName == null) {
            System.out.println("No log file location provided");
        }
        else {
            try {
                _psLog = new PrintStream(new FileOutputStream(strFileName));
            }
            catch (FileNotFoundException fnfe) {
            }
        }
        
        // Write header to the file
        writeHeader();
    }
    
    /**
     * @see FanListener#addFan(Fan)
     */
    public void addFan(Fan f)
    {
        _fans.add(f);
    }

    /**
     * @see TempSensorListener#addTempSensor(TempSensor)
     */
    public void addTempSensor(TempSensor sensor)
    {
        _sensors.add(sensor);
    }

    /**
     * @see Module#update()
     */
    public void update()
    {
        // Create a buffer to write this out to
        StringBuffer buf = new StringBuffer(1024);
        
        // Add fan information to the log
        Iterator iterFans = _fans.iterator();
        while (iterFans.hasNext()) {
            Fan f = (Fan) iterFans.next();
            buf.append(f.getCurrentRPM());
            buf.append(':');
            buf.append(f.getCurrentPowerLevel());
            buf.append(':');
        }
        
        // Add sensor information to the log
        Iterator iterSensors = _sensors.iterator();
        while (iterSensors.hasNext()) {
            TempSensor s = (TempSensor) iterSensors.next();
            buf.append(s.getCurrentTemperature());
            buf.append(':');
        }
        
        // Now write out to disk
        synchronized (_psLog) {
            _psLog.println(buf.toString());
        }
    }

    /**
     * @see Module#getUpdateInterval()
     */
    public int getUpdateInterval()
    {
        return 5000;
    }
    
    private void writeHeader()
    {
        
    }
}
