/*
 * Copyright Ian Burnett 2005, 2006.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 *
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

package com.ianburnett.iilc.modules;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import com.ianburnett.iilc.Module;
import com.ianburnett.iilc.display.DisplayController;
import com.ianburnett.iilc.display.TextArea;
import com.ianburnett.iilc.display.Window;
import com.ianburnett.iilc.impl.ModuleImpl;

/**
 * Provides a simple module implementation which displays the current
 * time in a format of the user's choosing. The format must conform
 * to the specification laid out by the rules in the
 * {@link SimpleDateFormat} documentation and be set using the module
 * option "format". The default pattern used is "yyyy-MM-dd HH:mm:ss".
 * 
 * @author Ian Burnett
 *
 */
public class Time extends ModuleImpl implements DisplayController
{
    /* VARIOUS MODULE CONSTANTS ------------------------------------------- */

    /**
     * Default date format.
     */
    private final String _strDefaultFormat = "yyyy-MM-dd HH:mm:ss";
    
    /**
     * Name of the option used to control the 
     */
    private final String _strOptionName = "format";
    
    /* WORKING DATA ------------------------------------------------------- */

    /**
     * The set of windows we are controlling.
     */
    private final Set _windows = new HashSet();
    
    /**
     * The date formatting object to use.
     */
    private DateFormat _fmt;
    
    
    /* MAIN MODULE LOGIC -------------------------------------------------- */

    /**
     * @see Module#update()
     */
    public void update()
    {
        // Get the current time and format
        String strDate = _fmt.format(new Date());
        
        // Set the text in each of our windows to be the current time
        Iterator iter = _windows.iterator();
        while (iter.hasNext()) {
            TextArea ta = (TextArea) iter.next();
            ta.setText(0, strDate);
        }
    }
    
    /* SETUP ROUTINES ----------------------------------------------------- */

    /**
     * @see Module#init()
     */
    public void init()
    {
        // Initialise superclass first
        super.init();
        
        // Try to get the desired time format
        String df = getOption(_strOptionName);
        if (df == null) {
            df = _strDefaultFormat;
        }
        
        // Initialse the date format object
        _fmt = new SimpleDateFormat(df);
    }
    
    /**
     * @see DisplayController#addDisplayWindow(Window)
     */
    public void addDisplayWindow(Window window)
    {
        // Get dimensions
        int w = window.getWidth(), h = window.getHeight();
        
        // Initialise to have a single line of text at (0,0)
        TextArea ta = window.createTextArea(0, 0, w, h);
        
        // Add this to the set of text areas we update
        this._windows.add(ta);
    }
    
    /**
     * @see Module#getUpdateInterval()
     */
    public int getUpdateInterval()
    {
        // Once a second is enough for this module
        return 1000;
    }
    
}
