/*
 * Copyright Ian Burnett 2005, 2006.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 *
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

package com.ianburnett.iilc.server;

import com.ianburnett.iilc.server.parser.ASTControlDirective;
import com.ianburnett.iilc.server.parser.ASTStringLiteral;
import com.ianburnett.iilc.server.parser.Node;

class ConfigControl
{
    /**
     * The name of the module in the config file to receive control
     * requests.
     */
    private String _strControlledModule;
    
    /**
     * The name of the device in the controlled module.
     */
    private String _strControlledName;

    /**
     * The definition in the config tree this control request refers to.
     */
    private ConfigDefine _cfgDefine;
    
    /**
     * Converts an AST object into a configuration object.
     * 
     * @param astControl
     */
    public ConfigControl(ASTControlDirective astControl)
    {
        // Get all the children objects
        Node child1 = astControl.jjtGetChild(0);
        Node child2 = astControl.jjtGetChild(1);

        // First object is the module which we are targeting
        _strControlledModule = ((ASTStringLiteral) child1).toString();
        
        // Second child is the object within that module
        _strControlledName = ((ASTStringLiteral) child2).toString();
    }

    /**
     * Retrieves the definition this control object refers to.
     * 
     * @return
     */
    public ConfigDefine getDefinition()
    {
        return _cfgDefine;
    }
    
    /**
     * Performs checks to ensure this object is correctly configured.
     * 
     * @param cfgFile
     * @param cfgModule
     * 
     * @return true to indicate success, false otherwise.
     */
    public boolean validate(ConfigFile cfgFile, ConfigModule cfgModule)
    {
        // Look up the definition in the main file
        ConfigModule cfgTargetModule = cfgFile.getModule(_strControlledModule);
        if (cfgTargetModule == null) {
            cfgFile.warning("Module '" + _strControlledModule +
                    "' does not exist in the config file - ignored");
            return false;
        }
        
        // Look up the definition in the module
        _cfgDefine = cfgTargetModule.getEntityDefinition(_strControlledName);
        if (_cfgDefine == null) {
            cfgFile.error("No such definition " + _strControlledName +
                    " in module " + _strControlledModule);
            return false;
        }
        
        // Now need to check if this definition is already being controlled
        ConfigModule cfgModuleController = _cfgDefine.getControllingModule();
        if (cfgModuleController != null) {
            cfgFile.warning("Module '" + cfgModuleController.getName() +
                    "' already controls the entity - ignored");
            return false;
        }
        else {
            // Mark our parent module as controlling this definition
            _cfgDefine.setControllingModule(cfgModule);
        }
        
        // All ok
        return true;
    }
    
}
