/*
 * Copyright Ian Burnett 2005, 2006.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 *
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

package com.ianburnett.iilc.server;

import com.ianburnett.iilc.entity.Entity;
import com.ianburnett.iilc.server.parser.ASTDefineDirective;
import com.ianburnett.iilc.server.parser.ASTEntityFan;
import com.ianburnett.iilc.server.parser.ASTEntityLED;
import com.ianburnett.iilc.server.parser.ASTEntityTemp;
import com.ianburnett.iilc.server.parser.ASTIntegerLiteral;
import com.ianburnett.iilc.server.parser.ASTStringLiteral;
import com.ianburnett.iilc.server.parser.Node;

/**
 * Represents the definition of a entity in the configuration file. One of
 * these objects are created for every occurrence of the keyword "define"
 * in the configuration file.
 *  
 * @author Ian Burnett
 */
class ConfigDefine
{
    public static final int DEFINE_TYPE_FAN = 1;
    
    public static final int DEFINE_TYPE_LED = 2;
    
    public static final int DEFINE_TYPE_TEMP = 3;

    private String _name;
    
    private int _defineType;
    
    private int _id;
    
    private ConfigModule _controllingModule;
    
    private Entity _objInstance;
    
    public ConfigDefine(ASTDefineDirective astDefine)
    {
        // Setup correct initial state
        this._name = null;
        this._defineType = 0;
        this._id = 0;
        this._controllingModule = null;
        
        // Get all child objects we can
        Node child1 = astDefine.jjtGetChild(0);
        Node child2 = astDefine.jjtGetChild(1);
        Node child3 = astDefine.jjtGetChild(2);
        
        // Grab the required information
        if (child1 instanceof ASTEntityFan) {
            _defineType = DEFINE_TYPE_FAN;
        }
        else if (child1 instanceof ASTEntityLED) {
            _defineType = DEFINE_TYPE_LED;
        }
        else if (child1 instanceof ASTEntityTemp) {
            _defineType = DEFINE_TYPE_TEMP;
        }
        
        // Child 2 is the raw device ID
        _id = Integer.parseInt(((ASTIntegerLiteral) child2).toString());
        
        // Child 3 is the name of the definition
        _name = ((ASTStringLiteral) child3).toString();
    }
    
    public String getName()
    {
        return _name;
    }

    public ConfigModule getControllingModule()
    {
        return _controllingModule;
    }
    
    public int getDefineType()
    {
        return _defineType;
    }
    
    public void setControllingModule(ConfigModule cfgModule)
    {
        this._controllingModule = cfgModule;
    }
    
    public Entity getInstance()
    {
        if (_objInstance != null) {
            return _objInstance;
        }
        
        switch ( _defineType ) {
        
        case ConfigDefine.DEFINE_TYPE_FAN:
            _objInstance = new ControlledFanImpl(_id, _name);
            break;
            
        case ConfigDefine.DEFINE_TYPE_LED:
            _objInstance = new ControlledLEDImpl(_id, _name);
            break;
            
        case ConfigDefine.DEFINE_TYPE_TEMP:
            _objInstance = new TempImpl(_id, _name);
            break;
        }
        
        // Return to caller
        return _objInstance;
    }
}


