/*
 * Copyright Ian Burnett 2005, 2006.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 *
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

package com.ianburnett.iilc.server;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.ianburnett.iilc.InvalidConfigurationException;
import com.ianburnett.iilc.Module;
import com.ianburnett.iilc.display.DisplayProvider;
import com.ianburnett.iilc.server.parser.ASTDisplayDecl;
import com.ianburnett.iilc.server.parser.ASTScreenDecl;
import com.ianburnett.iilc.server.parser.ASTStringLiteral;
import com.ianburnett.iilc.server.parser.Node;

class ConfigDisplay
{
    /**
     * Name of the display configured.
     */
    private String _name;
    
    /**
     * The name of the module to use. 
     */
    private String _moduleName;
    
    /**
     * The list of screens we are to control.
     */
    private List _cfgScreens;

    /**
     * 
     */
    private ConfigModule _cfgModule;
    
    private DisplayImpl _display;
    
    /**
     * 
     * @param astDisplay
     */
    public ConfigDisplay(ASTDisplayDecl astDisplay)
    {
        // Init storage
        _cfgScreens = new ArrayList();
        
        // First child is always the name
        Node child1 = astDisplay.jjtGetChild(0);
        this._name = ((ASTStringLiteral) child1).toString();
        
        // Second child is always the device to use
        Node child2 = astDisplay.jjtGetChild(1);
        this._moduleName = ((ASTStringLiteral) child2).toString();

        // Each subsequent child is a screen declaration
        int numChildren = astDisplay.jjtGetNumChildren();
        for (int i = 2; i < numChildren; i++) {

            // Get the node itself
            Node n = astDisplay.jjtGetChild(i);
            
            // Decide on the type and the action
            if (n instanceof ASTScreenDecl) {
                
                // Add to the list of screens this display controls
                ConfigScreen cfgScreen = new ConfigScreen((ASTScreenDecl) n);
                _cfgScreens.add(cfgScreen);
            }
        }
        
    }

    public void validate(ConfigFile cfgFile)
    {
        // Get the module we need to control a screen
        _cfgModule = cfgFile.getModule(_moduleName);
        if (_cfgModule == null) {
            cfgFile.warning("Could not locate the module " + _moduleName);
        }
        
        // Validate each of the screens
        Iterator iter = _cfgScreens.iterator();
        while (iter.hasNext()) {
            ConfigScreen cfgScreen = (ConfigScreen) iter.next();
            cfgScreen.validate(cfgFile);
        }
    }

    public DisplayImpl getInstance()
            throws InvalidConfigurationException
    {
        // Get the module which will provide the physical screen
        Module module = _cfgModule.getInstance();
        
        // Create a new display based on this module
        _display = new DisplayImpl((DisplayProvider) module);
        
        // Get an object instance for each screen this display contains
        Iterator iter = _cfgScreens.iterator();
        while (iter.hasNext()) {
            ConfigScreen cfgScreen = (ConfigScreen) iter.next();
            ScreenImpl screen = cfgScreen.getInstance();
            _display.addScreen(screen);
        }
        
        // Return the configured display
        return _display;
    }
    
    
    public String getName()
    {
        return _name;
    }
}


