/*
 * Copyright Ian Burnett 2005, 2006.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 *
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

package com.ianburnett.iilc.server;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import com.ianburnett.iilc.InvalidConfigurationException;
import com.ianburnett.iilc.Module;
import com.ianburnett.iilc.entity.Entity;
import com.ianburnett.iilc.entity.fan.ControlledFan;
import com.ianburnett.iilc.entity.fan.Fan;
import com.ianburnett.iilc.entity.fan.FanController;
import com.ianburnett.iilc.entity.fan.FanListener;
import com.ianburnett.iilc.entity.fan.FanProvider;
import com.ianburnett.iilc.entity.led.ControlledLED;
import com.ianburnett.iilc.entity.led.LEDController;
import com.ianburnett.iilc.entity.led.LEDProvider;
import com.ianburnett.iilc.entity.temp.TempSensor;
import com.ianburnett.iilc.entity.temp.TempSensorListener;
import com.ianburnett.iilc.entity.temp.TempSensorProvider;
import com.ianburnett.iilc.server.parser.ASTControlDirective;
import com.ianburnett.iilc.server.parser.ASTDefineDirective;
import com.ianburnett.iilc.server.parser.ASTModuleDecl;
import com.ianburnett.iilc.server.parser.ASTOptionDirective;
import com.ianburnett.iilc.server.parser.ASTReportDirective;
import com.ianburnett.iilc.server.parser.ASTStringLiteral;
import com.ianburnett.iilc.server.parser.Node;

/**
 * 
 * @author Ian Burnett
 */
class ConfigModule
{
    private String _name;
    
    private String _implStrClass;
    
    private Class _implClass;
    
    private Constructor _defaultConstructor;
    
    private Map _cfgDefines;
    
    private Set _cfgReports;
    
    private Set _cfgOptions;
    
    private Set _cfgControls;
    
    private Module _module;
    
    /**
     * Constructor which takes the AST object and retrieves all necessary
     * information from it.
     * 
     * @param astModule The AST object used to initialise this object.
     */
    public ConfigModule(ASTModuleDecl astModule)
    {
        // Setup storage
        this._module = null;
        this._cfgDefines = new HashMap();
        this._cfgReports = new HashSet();
        this._cfgOptions = new HashSet();
        this._cfgControls = new HashSet();

        // First child is always the name
        Node child1 = astModule.jjtGetChild(0);
        this._name = ((ASTStringLiteral) child1).toString();
        
        // Second child is always the class to use
        Node child2 = astModule.jjtGetChild(1);
        this._implStrClass = ((ASTStringLiteral) child2).toString();
           
        // Walk over each remaining child entry we find
        int numChildren = astModule.jjtGetNumChildren();
        for (int i = 2; i < numChildren; i++) {

            // Get the node itself
            Node n = astModule.jjtGetChild(i);
            
            // Decide on the type and the action
            if (n instanceof ASTControlDirective) {
                // Add to the set of items this module controls
                ConfigControl cfgControl = new ConfigControl((ASTControlDirective) n);
                _cfgControls.add(cfgControl);
            }
            else if (n instanceof ASTReportDirective) {
                // Add to the set of items this module receives reports for 
                ConfigReport cfgReport = new ConfigReport((ASTReportDirective) n);
                _cfgReports.add(cfgReport);
            }
            else if (n instanceof ASTOptionDirective) {
                // Add to the set of data this module is configured with
                ConfigOption cfgOption = new ConfigOption((ASTOptionDirective) n);
                _cfgOptions.add(cfgOption);
            }
            else if (n instanceof ASTDefineDirective) {
                // Add to the map of items this module orignates data for
                ConfigDefine cfgDefine = new ConfigDefine((ASTDefineDirective) n);
                _cfgDefines.put(cfgDefine.getName(), cfgDefine);
            }
        }
    }
    
    public String getName()
    {
        return _name;
    }
    
    public String getImplClass()
    {
        return _implStrClass;
    }
    
    public ConfigDefine getEntityDefinition(String name)
    {
        return (ConfigDefine) _cfgDefines.get(name);
    }
    
    public Set getConfigOptions()
    {
        return _cfgOptions;
    }
    
    public void validate(ConfigFile cfgFile)
    {
        /*
         * Preload the class definition and check this is OK
         */
        
        try {
            // Load the class
            _implClass = Class.forName(_implStrClass);
        
            // Verify a descendant of IILCModule
            if ( ! Module.class.isAssignableFrom(_implClass) ) {
                cfgFile.error("Class " + _implStrClass +
                        " not a valid IILC module type");
                return;
            }
        
            // Find the default constructor
            _defaultConstructor = _implClass.getConstructor(new Class[0]);
        }
        catch (NoSuchMethodException nsme) {
            System.out.println(nsme);
            // getConstructor
        }
        catch (ClassNotFoundException cnfe) {
            cfgFile.error("Could not find the module class '" + 
                    _implStrClass + "'");
            return;
        }
        catch (UnsatisfiedLinkError ule) {
            System.out.println(ule);
            cfgFile.error("Could not load the native " +
                    "library for the device " + _name);
            return;
        }
        
        // Validate each report entry, removing on error
        Iterator iterReports = _cfgReports.iterator();
        while (iterReports.hasNext()) {
            ConfigReport cfgReport = (ConfigReport) iterReports.next();
            if ( ! cfgReport.validate(cfgFile, this) ) {
                iterReports.remove();
            }
        }
        
        // Validate each control entry, removing on error
        Iterator iterControls = _cfgControls.iterator();
        while (iterControls.hasNext()) {
            ConfigControl cfgControl = (ConfigControl) iterControls.next();
            if ( ! cfgControl.validate(cfgFile, this) ) {
                iterControls.remove();
            }
        }
        
        // Now the same again for each definition entry

        Iterator iterDefineNames = _cfgDefines.keySet().iterator();
        while (iterDefineNames.hasNext()) {
            
            // Get the name of the definition
            String strDefineName = (String) iterDefineNames.next();
            ConfigDefine cfgDefine = (ConfigDefine) _cfgDefines.get(strDefineName);

            // Does this implementation support the definition of this type?
            switch (cfgDefine.getDefineType()) {
            
            case ConfigDefine.DEFINE_TYPE_FAN:
                if ( ! FanProvider.class.isAssignableFrom(_implClass) ) {
                    cfgFile.warning("Module '" + _name +
                            "' not declared to provide fan support - ignoring");
                    iterDefineNames.remove();
                    continue;
                }
                break;
                
            case ConfigDefine.DEFINE_TYPE_LED:
                if ( ! LEDProvider.class.isAssignableFrom(_implClass) ) {
                    cfgFile.warning("Module '" + _name +
                            "' not declared to provide fan support - ignoring");
                    iterDefineNames.remove();
                    continue;
                }
                break;
                
            case ConfigDefine.DEFINE_TYPE_TEMP:
                if ( ! TempSensorProvider.class.isAssignableFrom(_implClass) ) {
                    cfgFile.warning("Module '" + _name +
                            "' not declared to provide temperature support - ignoring");
                    iterDefineNames.remove();
                    continue;
                }
                break;
            }
        }
    }
    
    public Module getInstance() throws InvalidConfigurationException
    {
        // The module to return
        if (_module != null) {
            return _module;
        }
        
        try {
            // Create an instance of the implementation class
            _module = (Module) _defaultConstructor.newInstance(null);
        }
        catch (InvocationTargetException ite) {
            System.out.println(ite);
            Throwable t = ite.getTargetException();
            t.printStackTrace(System.out);
            // newInstance
        }
        catch (IllegalAccessException iae) {
            System.out.println(iae);
            // newinstance
        }
        catch (InstantiationException ie) {
            System.out.println(ie);
            // newinstance
        }
        
        // Set each option specified on the module
        Iterator iterOpts = _cfgOptions.iterator();
        
        // Get each option & set on the module
        while (iterOpts.hasNext()) {
            ConfigOption cfgOption = (ConfigOption) iterOpts.next();
            _module.setOption(cfgOption.getKey(), cfgOption.getValue());
        }
        
        // Tell this module about all the reports it is to receive
        Iterator iterReports = _cfgReports.iterator();
        while (iterReports.hasNext()) {
            
            // Get the configuration details
            ConfigReport cfgReport = (ConfigReport) iterReports.next();
            ConfigDefine cfgDefine = cfgReport.getDefinition();
            
            // Create an instance
            Entity e = cfgDefine.getInstance();
            
            // Decide on the sort of definition this is
            switch (cfgDefine.getDefineType()) {
            
            case ConfigDefine.DEFINE_TYPE_FAN:
                ((FanListener) _module).addFan((Fan) e);
                break;
                
            case ConfigDefine.DEFINE_TYPE_LED:
                // No such thing!
                break;
                
            case ConfigDefine.DEFINE_TYPE_TEMP:
                ((TempSensorListener) _module).addTempSensor((TempSensor) e);
                break;
            }
        }
        
        // Tell the module about each definition it controls
        Iterator iterControls = _cfgControls.iterator();
        while (iterControls.hasNext()) {
            
            // Get the configuration details
            ConfigControl cfgControl = (ConfigControl) iterControls.next();
            ConfigDefine cfgDefine = cfgControl.getDefinition();
            Entity e = cfgDefine.getInstance();
            
            // Decide on what sort of definition this is
            switch (cfgDefine.getDefineType()) {
            
            case ConfigDefine.DEFINE_TYPE_FAN:
                ((FanController) _module).addControlledFan((ControlledFanImpl) e);
                break;
            
            case ConfigDefine.DEFINE_TYPE_LED:
                ((LEDController) _module).addControlledLED((ControlledLED) e);
                break;
                
            case ConfigDefine.DEFINE_TYPE_TEMP:
                // No such thing!
                 break;
            }
        }
        
        
        // Tell this module about each definition against itself
        Iterator iterDefines = _cfgDefines.values().iterator();
        while (iterDefines.hasNext()) {
            
            // Get the configuration details
            ConfigDefine cfgDefine = (ConfigDefine) iterDefines.next();
            Entity e = cfgDefine.getInstance();
            
            switch (cfgDefine.getDefineType()) {
            
            // Tell the fan about the module; and the module about the fan
            case ConfigDefine.DEFINE_TYPE_FAN:
                
                // Cast the entity to the correct type
                ControlledFanImpl fanImpl = (ControlledFanImpl) e;
                FanProvider fanProvider = (FanProvider) _module;
                
                // Wrapper the controlled fan to avoid having the ability
                // to cast accidentally
                fanProvider.addMonitoredFan(new FanImpl(fanImpl));
                fanImpl.setFanProvider(fanProvider);
                break;
                
            case ConfigDefine.DEFINE_TYPE_LED:
                ControlledLEDImpl     ledImpl     = (ControlledLEDImpl) e;
                LEDProvider ledProvider = (LEDProvider) _module;
                ledProvider.addMonitoredLED(ledImpl);
                ledImpl.setProvider(ledProvider);
                break;
                
            case ConfigDefine.DEFINE_TYPE_TEMP:
                TempImpl     tempImpl     = (TempImpl) e;
                TempSensorProvider tempProvider = (TempSensorProvider) _module;
                tempProvider.addMonitoredTempSensor(tempImpl);
                tempImpl.setProvider(tempProvider);
                break;
            }
        }
        
        // Return the constructed module
        return _module;
    }
    
}

