/*
 * Copyright Ian Burnett 2005, 2006.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 *
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

package com.ianburnett.iilc.server;

import com.ianburnett.iilc.InvalidConfigurationException;
import com.ianburnett.iilc.Module;
import com.ianburnett.iilc.display.DisplayController;
import com.ianburnett.iilc.display.DisplayProvider;
import com.ianburnett.iilc.server.parser.ASTDimension;
import com.ianburnett.iilc.server.parser.ASTIntegerLiteral;
import com.ianburnett.iilc.server.parser.ASTLocation;
import com.ianburnett.iilc.server.parser.ASTStringLiteral;
import com.ianburnett.iilc.server.parser.ASTWindowDecl;
import com.ianburnett.iilc.server.parser.Node;

class ConfigWindow
{
    /**
     * Name of the module to use.
     */
    private String _moduleName = null;
    
    /**
     * The module used to control the display.
     */
    private ConfigModule _cfgModule = null;
    
    /**
     * Width of the module window to display. Zero means whole width available.
     */
    private int _w = 0;
    
    /**
     * Height of the module window to display. Zero means whole height available.
     */
    private int _h = 0;
    
    /**
     * Location of the top left-hand corner of the window. Co-ordinates are zero-based
     * and count left to right.
     */
    private int _x = 0;
    
    /**
     * Location of the top left-hand corner of the window. Co-ordinates are zero-based
     * and count top to bottom.
     */
    private int _y = 0;

    private WindowImpl _window;
    
    ConfigWindow(ASTWindowDecl astWindow)
    {
        // All of this information may be present
        ASTStringLiteral astStringLiteral = null;
        ASTLocation astLocation = null;
        ASTDimension astDimension = null;

        // Save repeated method calls
        int numChildren = astWindow.jjtGetNumChildren();
        
        // First child is the name of the module to use
        Node child1 = astWindow.jjtGetChild(0);
        astStringLiteral = (ASTStringLiteral) child1;

        // Next child may be present
        if (numChildren > 1) {
            Node child2 = astWindow.jjtGetChild(1);
            if (child2 instanceof ASTLocation) {
                astLocation = (ASTLocation) child2;
            }
            else if (child2 instanceof ASTDimension) {
                astDimension = (ASTDimension) child2;
            }
        }
        
        // Third child may also be present
        if (numChildren > 2) {
            Node child3 = astWindow.jjtGetChild(2);
            if (child3 instanceof ASTLocation) {
                astLocation = (ASTLocation) child3;
            }
            else if (child3 instanceof ASTDimension) {
                astDimension = (ASTDimension) child3;
            }
        }
        
        // Now store the information
        this._moduleName = astStringLiteral.toString();
        
        // May have a location
        if (astLocation != null) {
            ASTIntegerLiteral x = (ASTIntegerLiteral) astLocation.jjtGetChild(0);
            ASTIntegerLiteral y = (ASTIntegerLiteral) astLocation.jjtGetChild(1);
            this._x = Integer.parseInt(x.toString());
            this._y = Integer.parseInt(y.toString());
        }
        
        // May have a size
        if (astDimension != null) {
            ASTIntegerLiteral w = (ASTIntegerLiteral) astDimension.jjtGetChild(0);
            ASTIntegerLiteral h = (ASTIntegerLiteral) astDimension.jjtGetChild(1);
            this._w = Integer.parseInt(w.toString());
            this._h = Integer.parseInt(h.toString());
        }
    }

    
    public void validate(ConfigFile cfgFile)
    {
        // Get the module we need to control a screen
        _cfgModule = cfgFile.getModule(_moduleName);
        if (_cfgModule == null) {
            cfgFile.warning("Could not locate the module " + _moduleName);
        }
        
    }

    public WindowImpl getInstance()
            throws InvalidConfigurationException
    {
        
        // Get the module which will control the display 
        Module module = _cfgModule.getInstance();
        if ( ! (module instanceof DisplayController) ) {
            throw new InvalidConfigurationException("Module not instance of display controller");
        }
        
        // Cast to correct type
        DisplayController dispController = (DisplayController) module;
        
        // Create a new window based on this module
        _window = new WindowImpl(_x, _y, _h, _w);
        
        // Tell the display controller about the window
        dispController.addDisplayWindow(_window);
        
        // Return the configured window
        return _window;
    }


    /**
     * @return the _h
     */
    public int getHeight()
    {
        return _h;
    }


    /**
     * @return the _w
     */
    public int getWidth()
    {
        return _w;
    }


    /**
     * @return the _x
     */
    public int getX()
    {
        return _x;
    }


    /**
     * @return the _y
     */
    public int getY()
    {
        return _y;
    }

}
