/*
 * Copyright Ian Burnett 2005, 2006.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 *
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

package com.ianburnett.iilc.server;

import com.ianburnett.iilc.entity.fan.ControlledFan;
import com.ianburnett.iilc.entity.fan.Fan;
import com.ianburnett.iilc.entity.fan.FanProvider;

/**
 * Internal implementation of the ControlledFan interface.
 *  
 * @author Ian Burnett
 */
class ControlledFanImpl extends EntityImpl implements ControlledFan
{
    /**
     * The current fan speed as last reported to this object.
     */
    private int _lastReportedRPM = Fan.SPEED_STOPPED;

    /**
     * The object which provides this fan.
     */
    private FanProvider _fp = null;
    
    /**
     * The current power level this fan is set to.
     */
    private int _powerLevel = -1;
    
    /**
     * 
     * @param deviceID
     * @param name
     */
    ControlledFanImpl(int deviceID, String name)
    {
        super(deviceID, name);
    }
    
    /**
     * @see Fan#getCurrentPowerLevel()
     */
    public int getCurrentPowerLevel()
    {
        return _powerLevel;
    }

    /**
     * @see Fan#getCurrentRPM()
     */
    public int getCurrentRPM()
    {
        return _lastReportedRPM;
    }

    /**
     * @see Fan#reportFanSpeed(int)
     */
    public void reportFanSpeed(int rpm)
    {
        // Ignore anything not in sensible range
        if (rpm > 0) {
            _lastReportedRPM = rpm;
        }
        else {
            switch (rpm) {
            case Fan.SPEED_STOPPED:
            case Fan.SPEED_TOO_LOW:
            case Fan.SPEED_TOO_HIGH:
            case Fan.SPEED_UNKNOWN:
                // All acceptable
                break;
                
            default:
                // Should silently ignore this value
            }
        }
    }

    /**
     * @see ControlledFan#setFanPowerLevel(int)
     */
    public void setFanPowerLevel(int percentage)
    {
        // Ignore silly values
        if (percentage < 0 || percentage > 100) {
            return;
        }
        
        // Don't update if not a change in value
        if (percentage != _powerLevel) {
            _powerLevel = percentage;
            _fp.setFanPower(getID(), percentage);
        }
    }

    /**
     * Package private method to hook this fan instance up with
     * its provider.
     *  
     * @param fp A valid FanProvider object
     */
    void setFanProvider(FanProvider fp)
    {
        this._fp = fp;
    }    
}

