/*
 * Copyright Ian Burnett 2005, 2006.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 *
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

package com.ianburnett.iilc.server;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import com.ianburnett.iilc.display.DisplayProvider;
import com.ianburnett.iilc.entity.keypad.KeyListener;
import com.ianburnett.iilc.entity.keypad.KeyProvider;

/**
 * Internal implementation of the 
 * @author Ian Burnett
 *
 */
class DisplayImpl implements KeyListener
{
    /**
     * Flag to indicate if this instance has been initialised correctly.
     */
    private boolean _bInit = false;

    /**
     * Driver through which all comms should take place when talking
     * to the physical device.
     */
    private DisplayProvider _displayProvider = null;
    
    /**
     * 
     */
    private KeyProvider _keyProvider = null;
    
    /**
     * The time at which we should switch the backlight off. Zero
     * means that the backlight is already off.
     */
    private long _nBacklightOff = 0;

    /**
     * The index of the current screen being displayed.
     */
    private int _nCurrentScreen;
    
    /**
     * The list of screens this display is controlling.
     */
    private final List _screens = new ArrayList();

    /**
     * 
     * @param screenProvider
     */
    public DisplayImpl(DisplayProvider displayProvider)
    {
        // Store the display provider instance
        _displayProvider = displayProvider;
        
        // If this display provider also has a keypad, use that too
        _keyProvider = displayProvider.getKeyProvider();
        if (_keyProvider != null) {
            _keyProvider.addKeyListener(this);
        }
    }
    
    /**
     * 
     *
     */
    public void init()
    {
        // One-time initialisation
        if (_bInit) {
            return;
        }
        
        // Flag we are initialised
        _bInit = true;
    }
    
    /**
     * 
     *
     */
    public void start()
    {
        // Start by initialising the screen to sensible values
        _displayProvider.clearScreen();
        
        // Switch the backlight on now and turn off in 5 seconds
        _displayProvider.setBacklight(30);
        _nBacklightOff = new Date().getTime() + 5000;
        
        // Set the contrast to something legible
        _displayProvider.setContrast(90);
    }

    /**
     * 
     *
     */
    public void stop()
    {
        
    }

    /**
     * We would like to be updated once every 100ms.
     * 
     * @return The interval in milliseconds before we should be called
     * to update again.
     */
    public int getUpdateInterval()
    {
        return 100;
    }
    
    /**
     * Updates the current status of the display.
     *
     */
    public void update()
    {
        // Should we turn the backlight off?
        if (_nBacklightOff != 0) {
            if (new Date().getTime() > _nBacklightOff) {
                _displayProvider.setBacklight(0);
                _nBacklightOff = 0;
            }
        }
        
        // Get the currently-displayed screen
        ScreenImpl screen = (ScreenImpl) _screens.get(_nCurrentScreen);
        
        // Get the size of the display provider
        int w = _displayProvider.getScreenSizeX();
        int h = _displayProvider.getScreenSizeY();
        
        // Create a buffer to render into
        char[][] buffer = new char[h][w];
        
        // Render the screen into this buffer
        screen.render(buffer);
        
        // Display it
        for (int r = 0; r < h; r++) {
            _displayProvider.writeText(0, r, new String(buffer[r]));
        }
        
        
//    case Event.TYPE_TEXT_SCROLL:
//        scrollText();
//        nRequeueTime = 300;
//        break;
        
    }

    public void addScreen(ScreenImpl screen)
    {
        _screens.add(screen);
    }

    private void scrollText()
    { 

        /*
    char tmpBuf[1024];
    size_t strLen = -1;
    int dispWidth = 20;
    
    /* Amount of string to end *
    strLen = strlen(&longText[scrollPos]);

    if (strLen > (unsigned int) dispWidth) {
        sprintf(tmpBuf, "%.*s", dispWidth, &longText[scrollPos]);
    }
    else {
        sprintf(tmpBuf, "%s %.*s", &longText[scrollPos], dispWidth - strLen - 1, longText);
    }

    /* Write to the display *
    (pDisplay->pFnArray->WriteText)(pDisplay->pDriver, 0, 1, tmpBuf, (unsigned int) strlen(tmpBuf));

    /* Scroll and check within bounds *
    scrollPos++;
    if ((unsigned int) scrollPos >= strlen(longText)) {
        scrollPos = 0;
    }

    */
    }

    private String getKeyName(int key)
    {
        switch (key) {
        case KeyListener.KEY_UP:    return "Up";
        case KeyListener.KEY_DOWN:  return "Down";
        case KeyListener.KEY_LEFT:  return "Left";
        case KeyListener.KEY_RIGHT: return "Right";
        case KeyListener.KEY_YES:   return "Yes";
        case KeyListener.KEY_NO:    return "No";
        default:                    return "Unknown";
        }
    }

    /**
     * @see KeyListener#reportKeyDown(int)
     */
    public void reportKeyDown(int key)
    {
        switch (key) {
        
        case KeyListener.KEY_YES: {
            // YES key toggles backlight permanently on and permanently off
            if (_nBacklightOff > 0) {
                _displayProvider.setBacklight(0);
                _nBacklightOff = 0;
            }
            else {
                _displayProvider.setBacklight(30);
                _nBacklightOff = Long.MAX_VALUE;
            }
            break;
        }
        
        // NO key lights up backlight for a further 5 seconds 
        case KeyListener.KEY_NO: {
            if (_nBacklightOff > 0) {
                // Switch off immediately
                _displayProvider.setBacklight(0);
                _nBacklightOff = 0;
            }
            else {
                // Switch on for just 5 seconds
                _displayProvider.setBacklight(30);
                _nBacklightOff = new Date().getTime() + 5000;
            }
            break;
        }
        
        default:
            break;

        } // switch
    }

    /**
     * @see KeyListener#reportKeyRelease(int)
     */
    public void reportKeyRelease(int key)
    {
        // Ignore key releases
    }
}
