/*
 * Copyright Ian Burnett 2005, 2006.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 *
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

package com.ianburnett.iilc.server;

import java.io.File;
import java.io.FileInputStream;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import com.ianburnett.iilc.Module;
import com.ianburnett.iilc.server.parser.ASTConfigFile;
import com.ianburnett.iilc.server.parser.IILCParser;
import com.ianburnett.iilc.server.utils.Event;
import com.ianburnett.iilc.server.utils.EventQueue;

public class IILCServer implements Runnable
{
    /**
     * The config file on which this whole server instance is based.
     */
    private ConfigFile _cfgFile;
    
    /**
     * The thread which periodically polls each module for updates.
     */
    private Thread _thrModules;
    
    /**
     * Set of modules which this server is looking after.
     */
    private Set _iilcModules;
    
    /**
     * Set of displays which this server is looking after.
     */
    private Set _iilcDisplays;
    
    /**
     * 
     */
    private boolean _isRunning;
    
    /**
     * The queue containing all the modules which must be updated.
     */
    private EventQueue _eventQueue;
    
    /**
     * Constructor which stores the configuration file away.
     * 
     * @param cfgFile
     */
    private IILCServer(ConfigFile cfgFile)
    {
        // Store the config tree away
        this._cfgFile = cfgFile;
        this._iilcModules = new HashSet();
        this._iilcDisplays = new HashSet();
        this._eventQueue = new EventQueue();
    }


    private void init() throws Exception
    {
        // Make this object into a thread to look after the modules
        this._thrModules = new Thread(this);
        
        // Construct each module instance
        Set setModuleNames = _cfgFile.getModuleNames();
        Iterator iterModuleNames = setModuleNames.iterator();
        while (iterModuleNames.hasNext()) {

            // Get the next module name
            String moduleName = (String) iterModuleNames.next();

            // Get the module itself
            ConfigModule cfgModule = _cfgFile.getModule(moduleName);

            // Produce a full IILCModule instance for it
            Module iilcModule = cfgModule.getInstance();
            _iilcModules.add(iilcModule);
            iilcModule.init();

            // Add each module to the queue of things to update
            _eventQueue.enqueue(new Event(Event.TYPE_POLL_MODULE, iilcModule), 0);
        }
        
        // Construct each display instance
        Set setDisplayNames = _cfgFile.getDisplayNames();
        Iterator iterDisplayNames = setDisplayNames.iterator();
        while (iterDisplayNames.hasNext()) {
            
            // Get the next display name
            String displayName = (String) iterDisplayNames.next();
            
            // Get the display itself
            ConfigDisplay cfgDisplay = _cfgFile.getDisplay(displayName);
            
            // Produce a full IILCDisplay instance for it
            DisplayImpl iilcDisplay = cfgDisplay.getInstance();
            _iilcDisplays.add(iilcDisplay);
            iilcDisplay.init();

            // Add each display to the queue of things to update
            _eventQueue.enqueue(new Event(Event.TYPE_SCREEN_REFRESH, iilcDisplay), 0);
        }
    }

    private void start()
    {
        // Start each module
        Iterator iterModules = _iilcModules.iterator();
        while (iterModules.hasNext()) {
            ((Module) iterModules.next()).start();
        }
        
        // Start each display we have configured
        Iterator iterDisplay = _iilcDisplays.iterator();
        while (iterDisplay.hasNext()) {
            ((DisplayImpl) iterDisplay.next()).start();
        }
        
        // Start our thread to look after each module
        _isRunning = true;
        this._thrModules.start();
    }

    private void stop()
    {
        // Wait for our thread to terminate naturally
        _isRunning = false;
        try {
            System.out.println("Waiting for main IILC server thread to die");
            _thrModules.join();
        }
        catch (InterruptedException ie) {
            
        }
        
        // Stop each display 
        Iterator iterDisplay = _iilcDisplays.iterator();
        while (iterDisplay.hasNext()) {
            ((DisplayImpl) iterDisplay.next()).stop();
        }
        
        // Stop each module
        Iterator iterModules = _iilcModules.iterator();
        while (iterModules.hasNext()) {
            ((Module) iterModules.next()).stop();
        }
    }

    private void destroy()
    {

    }

    /**
     * @param args
     */
    public static void main(String[] args) throws Exception
    {
        String fullPath = "test.conf";

        File file = new File(fullPath);
        FileInputStream fis = new FileInputStream(file);
        String strFilename = file.getCanonicalPath();
        IILCParser parser = new IILCParser(fis);

        // Process the input configuration file
        ASTConfigFile astFile = parser.ConfigFile();
        //astFile.dump("--> ");

        // Create a new config tree, based on the file supplied and validate
        ConfigFile cfgFile = new ConfigFile(astFile);
        cfgFile.validate();

        // Now we can release the AST tree & parser to free some memory
        astFile = null;
        parser = null;

        // New instance of the server, based on this config
        IILCServer server = new IILCServer(cfgFile);

        // Initialise everything & start
        server.init();
        server.start();

        while (true) {
            Thread.sleep(10000);
        }

        // Stop the server process & cleanup
        // server.stop();
        // server.destroy();
    }


    /**
     * Provides the main logic for the continuous running of the server
     * process. This involves repeatedly polling each module we are
     * controlling, waiting until one needs servicing.
     */
    public void run()
    {
        while ( _isRunning ) {

            int requeueTime;
            
            // Dequeue an event from the queue
            Event e = _eventQueue.dequeue();
            
            // Unpack
            switch (e.getType()) {
            
            case Event.TYPE_POLL_MODULE:
                Module m = (Module) e.getObjectData();
                m.update();
                requeueTime = m.getUpdateInterval();
                break;
            
            case Event.TYPE_SCREEN_REFRESH:
                DisplayImpl d = (DisplayImpl) e.getObjectData();
                d.update();
                requeueTime = d.getUpdateInterval();
                break;
                
            default:
                requeueTime = 0;
                break;
            }
            
            // Requeue this event
            if (requeueTime > 0) {
                _eventQueue.enqueue(e, requeueTime);
            }
        }
    }
}



