/*
 * Copyright Ian Burnett 2005, 2006.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 *
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

package com.ianburnett.iilc.server;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import com.ianburnett.iilc.display.TextArea;
import com.ianburnett.iilc.display.Window;
import com.ianburnett.iilc.display.WindowRegion;

/**
 * 
 * @author Ian Burnett
 */
class WindowImpl implements Window
{
    private final int _height;
    
    private final int _width;
    
    private final int _x;
    
    private final int _y;
    
    private final Set _components = new HashSet();
    
    /**
     * 
     * @param height
     * @param width
     */
    WindowImpl(int x, int y, int height, int width)
    {
        _height = height;
        _width = width;
        _x = x;
        _y = y;
    }

    public int getX()
    {
        return _x;
    }
    
    public int getY()
    {
        return _y;
    }
    
    /**
     * 
     */
    public TextArea createTextArea(int x, int y, int w, int h)
    {
        // Sanity check
        if (x < 0 || y < 0) {
            throw new IllegalArgumentException("x < 0 || y < 0");
        }
        if (w < 1 || h < 1) {
            throw new IllegalArgumentException("w < 1 || h < 1");
        }
        
        // Check width is sensible
        if (x + w > _width) {
            throw new IllegalArgumentException(
                    "x (" + x + ") + w (" + w + ") > width (" + _width + ")");
        }
        
        // Same for height
        if (y + h > _height) {
            throw new IllegalArgumentException(
                    "y (" + y + ") + h (" + h + ") > height (" + _height + ")");
        }
        
        // Create a new region
        TextArea ta = new TextAreaImpl(w, h);
        
        // Create a WindowComponent to store this information
        WindowComponent wc = new WindowComponent(x, y, w, h, ta);
        
        // Add to our collection of regions
        _components.add(wc);
        
        // Return to the caller
        return ta;
    }

    /**
     * @see Window#getHeight()
     */
    public int getHeight()
    {
        return _height;
    }

    /**
     * @see Window#getWidth()
     */
    public int getWidth()
    {
        return _width;
    }

    /**
     * 
     * @param output
     */
    public void render(char[][] output)
    {
        // Iterate over each component
        Iterator iter = _components.iterator();
        while (iter.hasNext()) {
            
            // Get the component
            WindowComponent wc = (WindowComponent) iter.next();
            
            // Allocate a region to copy data into and blank out
            char[][] buf = new char[wc._h][wc._w];
            for (int r = 0; r < wc._h; r++) {
                for (int c = 0; c < wc._w; c++) {
                    buf[r][c] = ' ';
                }
            }
            
            // Tell the object to render into this buffer
            wc._r.render(buf);
            
            // Copy this rendered buffer into the master buffer
            for (int r = 0; r < wc._h; r++) {
                System.arraycopy(buf[r], 0, output[wc._y + r], wc._x, wc._w);
            }
        }
    }

    private class WindowComponent
    {
        private int _x;
        
        private int _y;
        
        private WindowRegion _r;
        
        private int _w;
        
        private int _h;
        
        private WindowComponent(int x, int y, int w, int h, WindowRegion r)
        {
            this._x = x;
            this._y = y;
            this._w = w;
            this._h = h;
            this._r = r;
        }
        
    }
}
