/*
 * Copyright Ian Burnett 2005, 2006.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 *
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

package com.ianburnett.iilc.server.utils;

import java.util.Date;

/**
 * Provides a synchronized queue structure, where events are enqueued
 * with a priority based on a time when the event should be read. The
 * dequeue operation blocks until an event is available and ready to
 * be removed. 
 *  
 * @author Ian Burnett
 */
public class EventQueue
{
    /**
     * Front of the queue.
     */
    private EventEntry _eeFirst;

    /**
     * Back of the queue.
     */
    private EventEntry _eeLast;

    /**
     * Number of elements in the queue.
     */
    private int _size = 0;

    /**
     * 
     * @param e
     * @param fireInMillis
     */
    public void enqueue(Event e, long fireInMillis)
    {
        // Create a new entry
        EventEntry ee = new EventEntry(e);

        // Get the current time
        ee._dequeueTime = new Date();

        // Set the time when this is to dequeue
        long nDequeueTime = ee._dequeueTime.getTime() + fireInMillis;
        ee._dequeueTime.setTime(nDequeueTime);

        synchronized (this) {

            // Start at the back of the queue
            EventEntry curr = _eeLast;

            // Work forwards until we find one with the same or higher priority
            while (curr != null) {

                // Get the time this one is to dequeue
                long currDequeue = curr._dequeueTime.getTime();

                // Decide if this is to dequeue first
                if (currDequeue <= nDequeueTime) {
                    break; // curr should dequeue before us - fall in line here
                }

                // Move forward in the queue
                curr = curr._eePrev;
            }

            // Point the new item to the current one
            ee._eePrev = curr;

            // Are we at the front?
            if (curr == null) {

                // Point the new item to the original front item
                ee._eeNext = _eeFirst;

                // Point the queue front to the new item
                _eeFirst = ee;
            }
            else {

                // Point the new one to the next one
                ee._eeNext = curr._eeNext;

                // Point the current item to the new one
                curr._eeNext = ee;
            }

            // Fix up the back of queue or next queue item
            if (ee._eeNext == null) {
                _eeLast = ee; // At back of queue
            }
            else {
                ee._eeNext._eePrev = ee;
            }

            // Signal something in the queue has changed
            this.notify();
        }
    }

    /**
     * 
     * @return
     */
    public Event dequeue()
    {
        EventEntry ee = null;

        synchronized (this) {

            // Wait until something is available at front of queue
            while (_eeFirst == null) {
                try {
                    this.wait();
                }
                catch (InterruptedException ie) {
                    System.out.println(ie);
                }
            }

            // Now something at front of queue

            // Get the current time
            long now = new Date().getTime();

            // When do we dequeue the next event?
            long dequeueTime = _eeFirst._dequeueTime.getTime();

            // Can we dequeue yet?
            if (dequeueTime > now) {

                // Can't dequeue yet. We need to wait until the required time
                // has
                // elapsed or something new joins the queue with a more urgent
                // need.

                try {
                    // Perform the wait
                    this.wait(dequeueTime - now);
                }
                catch (InterruptedException ie) {
                    System.out.println(ie);
                }

                // Now we've either timed-out, or something more urgent arrived.
                // In either case, dequeue immediately and return to user
            }
            else {
                // OK to dequeue immediately
            }

            // Grab the first event (don't forget front of queue may have
            // changed)
            ee = _eeFirst;

            // Fix up front of queue
            _eeFirst = ee._eeNext;

            // Fix up the last item as reqd.
            if (_eeFirst == null) {
                // Nothing left in the queue - fix up back
                _eeLast = null;
            }
            else {
                // Now the front item has nothing before it
                _eeFirst._eePrev = null;
            }
        }

        // Return the event itself
        return ee._event;
    }

    private void dumpQueue()
    {
        EventEntry ee = _eeFirst;

        System.out.println("Start Q dump");
        while (ee != null) {
            System.out.println(" --   " + ee._dequeueTime.getTime());
            ee = ee._eeNext;
        }
        System.out.println("Last  " + _eeLast._dequeueTime.getTime());
        ee = _eeLast;
        while (ee != null) {
            System.out.println(" --   " + ee._dequeueTime.getTime());
            ee = ee._eePrev;
        }
        System.out.println("First " + _eeFirst._dequeueTime.getTime());
        System.out.println("End Q dump");
    }

    private class EventEntry
    {
        private EventEntry(Event e)
        {
            this._event = e;
        }

        private Event _event;

        private Date _dequeueTime;

        private EventEntry _eeNext = null;

        private EventEntry _eePrev = null;

    }

}

