/*
 * Copyright Ian Burnett 2005, 2007.
 *
 * This file is part of Ian's Interactive LCD controller (IILC).
 *
 * IILC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * IILC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with IILC; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
 */

#if defined(__linux__)
#  include <errno.h>
#  include <termios.h>
#  include <fcntl.h>
#  include <string.h>
#  include <stdlib.h>
#  include <signal.h>
#  include <unistd.h>
#elif defined(WIN32)
#  include <windows.h>
#endif

#include <jni.h>
#include "com_ianburnett_iilc_drivers_crystalfontz_CFPacketSerial.h"


/*
 * This file still has lots of work to do on it, like making it handle
 * error conditions in a much cleaner way. There's also lots more 
 * flexibility required, in terms of baud rate, port options, etc.
 * At the minute it just works with a following wind, which is good
 * enough for basic development.
 */

/* Size of our read buffer */
#define SZ_READ_BUFFER  1024

/*
 * Various globals used to avoid the overhead of a field or method
 * lookup every time we enter it
 */

/* Global used to avoid having to lookup the class every time */
static jclass cls = NULL;

/* Global used to hold the method ID for the getOption(String) method */
static jmethodID midGetOption = NULL;

/* Global used to hold the method ID for the setLastError(long, String) method */
static jmethodID midSetLastErr = NULL;

/* Global used to hold the field ID for the _handle field */
static jfieldID fidHandle = NULL;

/**
 * Calls into the Java routine to set the last error code for this thread.
 * 
 * Note that this routine does not have any concept of threads - instead
 * this is left to the Java implementation for simplicity.
 *
 * See the Java docs for the setLastError(long, String) method for a
 * discussion on why this isn't a Java static.
 */
static
void
setLastError(
             JNIEnv * pEnv,
             jobject  jObject,
             long     nErrCode,
             char *   pszStrError
             );

/**
 * Calls the local setLastError routine, using the native environment last
 * error code. On Windows, this is available from GetLastError(); while on
 * POSIX, this is from errno and strerror().
 */
static
void
setLastErrorNative(
                   JNIEnv * pEnv,
                   jobject  jObject
                   );

/*
 * Initialises the serial device, based on the port name supplied.
 */

JNIEXPORT
void
JNICALL
Java_com_ianburnett_iilc_drivers_crystalfontz_CFPacketSerial_initialiseDevice (
            JNIEnv * pEnv,
            jobject  jObject,
            jstring  jStrPort)
{
    int           i;
    jlong         jnDevice = 0;
    char *        pszPort  = NULL;
    const jchar * jchPort  = NULL;
    jsize         strSz    = 0;
    jboolean      isCopy   = JNI_FALSE;

#if defined(__linux__)
    struct termios portset;
#elif defined(WIN32)
    HANDLE       hDevice;
    DCB          dcb;
    COMMTIMEOUTS ct;
    BOOL         bSuccess = FALSE;
#endif

    /* Look up the class type for this object (stored as a file static) */
    cls = (*pEnv)->GetObjectClass(pEnv, jObject);

    /* Lookup the method ID for the setLastError(int, String) method */
    midSetLastErr = (*pEnv)->GetMethodID(pEnv, cls,
        "setLastError", "(JLjava/lang/String;)V");
    if (midSetLastErr == 0) {
        goto end_of_function;
    }

    /* If we fail now, we can just call setLastError and jump out */

    /* Lookup the field id for the device handle */
    /* TODO: NLS */
    fidHandle = (*pEnv)->GetFieldID(pEnv, cls, "_handle", "J");
    if (fidHandle == 0) {
        setLastError(pEnv, jObject, -1, "Could not find the field '_handle'");
        goto end_of_function;
    }

    /* Retrieve the length of the port string */
    strSz = (*pEnv)->GetStringLength(pEnv, jStrPort);

    /* Allocate enough memory */
    pszPort = malloc(strSz + 1);
    memset(pszPort, 0, strSz+1);

    /* Get the chars */
    jchPort = (*pEnv)->GetStringChars(pEnv, jStrPort, &isCopy);

    /* Copy into the buffer, checking this is all ASCII */
    for (i = 0; i < strSz; i++) {
        jchar c = jchPort[i];
        if (c < 128) {
            pszPort[i] = (char) c;
        }
        else {
            /* TODO: NLS */
            printf("Non-ASCII char found!!\n");
            goto end_of_function;
        }
    }

    /* Release the Java chars */
    (*pEnv)->ReleaseStringChars(pEnv, jStrPort, jchPort);

#if defined(__linux__)

    /* Open the actual port */
    jnDevice = open(pszPort, O_RDWR | O_NOCTTY | O_NDELAY);
    if (jnDevice == -1) {
        setLastErrorNative(pEnv, jObject);
        goto end_of_function;
    }

    if (tcgetattr(jnDevice, &portset) == -1) {
        setLastErrorNative(pEnv, jObject);
        goto end_of_function;
    }

    cfmakeraw(&portset);
    portset.c_cflag |= 0;
    cfsetispeed(&portset, B115200);
    cfsetospeed(&portset, B115200);
    if (tcsetattr(jnDevice, TCSANOW, &portset) == -1) {
        setLastErrorNative(pEnv, jObject);
        goto end_of_function;
    }

#elif defined(WIN32)

    /* Open the actual port */
    hDevice = CreateFile(pszPort, GENERIC_READ | GENERIC_WRITE, 0,
        NULL, OPEN_EXISTING, 0, NULL);
    if (hDevice == INVALID_HANDLE_VALUE) {
        setLastErrorNative(pEnv, jObject);
        goto end_of_function;
    }

    /* Obtain a DCB for the port */
    bSuccess = GetCommState(hDevice, &dcb);
    if ( ! bSuccess ) {
        setLastErrorNative(pEnv, jObject);
        goto end_of_function;
    }

    /* Standard setup for the display */
    dcb.BaudRate      = CBR_115200;
    dcb.ByteSize      = 8;
    dcb.Parity        = NOPARITY;
    dcb.StopBits      = ONESTOPBIT;
    dcb.fRtsControl   = RTS_CONTROL_ENABLE;
    dcb.fDtrControl   = DTR_CONTROL_ENABLE;
    dcb.fAbortOnError = FALSE;

    /* Now set this new state */
    bSuccess = SetCommState(hDevice, &dcb);
    if ( ! bSuccess ) {
        setLastErrorNative(pEnv, jObject);
        goto end_of_function;
    }

    /* Setup timeout data */
    bSuccess = GetCommTimeouts(hDevice, &ct);
    if ( ! bSuccess ) {
        setLastErrorNative(pEnv, jObject);
        goto end_of_function;
    }

    /* Use non-blocking read on this device */
    ct.ReadIntervalTimeout        = MAXDWORD;
    ct.ReadTotalTimeoutMultiplier = 0;
    ct.ReadTotalTimeoutConstant   = 0;

    /* Set these timeout values */
    bSuccess = SetCommTimeouts(hDevice, &ct);
    if ( ! bSuccess ) {
        setLastErrorNative(pEnv, jObject);
        goto end_of_function;
    }

    /* Now all OK - return to caller */
    jnDevice = (jlong) hDevice;

#endif

    /* All OK now - set the handle */
    (*pEnv)->SetLongField(pEnv, jObject, fidHandle, jnDevice);

end_of_function:

    if (pszPort) free(pszPort);
    return;
}

/*
 * Allows the hardware device to be released from software control
 */

JNIEXPORT
void
JNICALL
Java_com_ianburnett_iilc_drivers_crystalfontz_CFPacketSerial_destroyDevice (
            JNIEnv * pEnv,
            jobject jObject,
            jlong jLongHandle)
{

    /* Implementation goes here... :-) */

}

/*
 * Writes a buffer of data to the hardware device
 */

JNIEXPORT
jlong
JNICALL
Java_com_ianburnett_iilc_drivers_crystalfontz_CFPacketSerial_writeData (
            JNIEnv *   pEnv,
            jobject    jObject,
            jlong      jLongHandle,
            jbyteArray jByteArray)
{
    jlong    jNumBytesWritten = 0;
    jbyte *  buf              = NULL;
    jboolean isCopy           = JNI_FALSE;
    size_t   count            = 0;

#if defined(__linux__)
    int     fd = (int) jLongHandle;
    ssize_t numBytesWritten;
#elif defined(WIN32)
    HANDLE hDevice = (HANDLE) jLongHandle;
    DWORD  numBytesWritten;
    BOOL   bSuccess = FALSE;
#endif


    /* Get a copy of the buffer to write */
    buf = (*pEnv)->GetByteArrayElements(pEnv, jByteArray, &isCopy);

    /* Find out the size of the buffer */
    count = (*pEnv)->GetArrayLength(pEnv, jByteArray);

#if defined(__linux__)

    /* Perform the write */
    numBytesWritten = write(fd, buf, count);

    /* Check for error */
    if (numBytesWritten == -1) {
        setLastErrorNative(pEnv, jObject);
    }

    /* Store away the number of bytes successfully written */
    jNumBytesWritten = numBytesWritten;

#elif defined(WIN32)

    /* Perform the write */
    bSuccess = WriteFile(hDevice, buf, (DWORD) count, &numBytesWritten, NULL);
    if ( ! bSuccess ) {
        setLastErrorNative(pEnv, jObject);
    }

    /* Store away the number of bytes successfully written */
    jNumBytesWritten = numBytesWritten;

#endif

    /* Release the data from the Java byte[] (no changes made) */
    (*pEnv)->ReleaseByteArrayElements(pEnv, jByteArray, buf, JNI_ABORT);

    /* Return the number of bytes which were actually written */
    return jNumBytesWritten;
}



/*
 * Reads a buffer-load of data from the device.
 */

JNIEXPORT
jbyteArray
JNICALL
Java_com_ianburnett_iilc_drivers_crystalfontz_CFPacketSerial_readData (
            JNIEnv * pEnv,
            jobject  jObject,
            jlong    jLongHandle,
            jobject  jObjErrno)
{

    /* Buffer to receive read data */
    jbyte buf[SZ_READ_BUFFER];

    /* The byte array to return */
    jbyteArray retArray = NULL;

    /* Number of bytes successfully read */
    jsize jNumBytesRead = 0;

    /* Number of bytes in our read buffer */
    size_t count = sizeof(buf);

#if defined(__linux__)

    ssize_t numBytesRead;
    int fd = (int) jLongHandle;

#elif defined(WIN32)

    HANDLE hDevice = (HANDLE) jLongHandle;
    DWORD dwErrors = 0;
    COMSTAT comStat;
    BOOL bSuccess = FALSE;
    DWORD dwNumBytesToRead = (DWORD) count;
    DWORD dwNumBytesRead = 0;

#endif

    /*
     * End of declarations
     */

#if defined(__linux__)

    /* Perform the read */
    numBytesRead = read(fd, buf, count);
    if (numBytesRead == -1) {
        
        switch (errno) {

        /* Ignore EAGAIN condition (we're non-blocking I/O) */
        case EAGAIN:    break;

        default:
            setLastErrorNative(pEnv, jObject);
            break;
        }
        jNumBytesRead = -1;
    }
    else {
        /* Save away the numbers of bytes successfully read */
        jNumBytesRead = (jsize) numBytesRead;
    }

#elif defined(WIN32)

    /* Find out how much data is available to be read */
    ClearCommError(hDevice, &dwErrors, &comStat);

    /* Do we have data to read? */
    if ( comStat.cbInQue > 0 ) {

        bSuccess = ReadFile(hDevice, &buf, dwNumBytesToRead, &dwNumBytesRead, NULL);
        if ( ! bSuccess ) {
            setLastErrorNative(pEnv, jObject);
        }
        else {
            jNumBytesRead = (jsize) dwNumBytesRead;
        }
    }
 
#endif

    /* Set the errno accordingly */
    if (jNumBytesRead <= 0) {

        /* Create an empty byte array */
        retArray = (*pEnv)->NewByteArray(pEnv, 0);
    }
    else {

        /* Create a new byte array */
        retArray = (*pEnv)->NewByteArray(pEnv, jNumBytesRead);

        /* Set the elements */
        (*pEnv)->SetByteArrayRegion(pEnv, retArray, 0, jNumBytesRead, buf);
    }

    /* Return the new array to the caller */
    return retArray;
}



void setLastError(JNIEnv * pEnv, jobject jObject, long nErrCode, char * pszStrError)
{
    /* Common declarations */
    jlong   jlErrorCode  = nErrCode;
    jstring jstrErrorMsg = NULL;

    /* Create a Java string */
    jstrErrorMsg = (*pEnv)->NewStringUTF(pEnv, pszStrError);

    /* Call the method */
    (*pEnv)->CallVoidMethod(pEnv, jObject, midSetLastErr, jlErrorCode, jstrErrorMsg);
}



void setLastErrorNative(JNIEnv * pEnv, jobject jObject)
{
#if defined(__linux__)

    long   nErrno    = 0;
    char * pszStrErr = NULL;

#elif defined(WIN32)

    DWORD  dwSuccess = 0;
    DWORD  dwErrno   = 0;
    LPVOID lpMsgBuf  = NULL;
    
#endif

    /*
     * End of declarations
     */

#if defined(__linux__)

    /* Get the error code */
    nErrno = errno;

    /* Get the system-formatted string */
    pszStrErr = strerror(errno);

    /* Call the other setLastError routine */
    setLastError(pEnv, jObject, nErrno, pszStrErr);

#elif defined(WIN32)
    
    /* Get the last error code */
    dwErrno = GetLastError();

    /* Format the message in the local language */
    dwSuccess = FormatMessage(
        FORMAT_MESSAGE_ALLOCATE_BUFFER |
            FORMAT_MESSAGE_FROM_SYSTEM |
            FORMAT_MESSAGE_IGNORE_INSERTS,
        NULL,
        dwErrno,
        MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
        (LPTSTR) &lpMsgBuf,
        0,
        NULL);

    if ( dwSuccess ) {

        /* Call the other setLastError routine */
        setLastError(pEnv, jObject, dwErrno, lpMsgBuf);

        /* Release the allocated buffer */
        LocalFree(lpMsgBuf);
    }
    else {
        /* Pathological case */
        printf("Failed while formatting error message "
                    "(errno = %d) - "
                    "GetLastError() now returns %d\n",
                    dwErrno, GetLastError());
    }

#endif

}

